classdef Sel < hgsetget
		
	%%-- Properties --%%
	
	properties (SetAccess=public)
		wl1
		wl2
		t1
		t2
	end
	
	properties (SetAccess=private)
		h=struct('figgui',[],'axes',struct('crosshairs',[],'spec',[],'kin',[]),'crosshairs',struct('wl1',[],'wl2',[],'t1',[],'t2',[],'pt11',[],'pt12',[],'pt21',[],'pt22',[]),'etxt',struct('wl1',[],'wl2',[],'t1',[],'t2',[]));
		opts=struct('etxtformat','%4.3f','nearpix',4,'line',struct('style','-','width',0.5,'color','white','pointer',struct('wl','left','t','top')),'pt',struct('color','k','linestyle','none','marker','+','size',6,'pointer','fleur'),'shadecolor',[0.95,0.95,0.95]);
	end
	
	properties (Dependent,SetAccess=private)
		wls
		wl_span
		ts
		t_span
		hs
		lims
	end
	
	
	
	%%-- Methods --%%
	
	%% Public %%
	methods
		
		%% Constructor
		function aSel=Sel(pt1,pt2,h_figGUI,varargin)
			
			%% Handle lack of arguments
			if(nargin==0)
				
				%% Warning
				warning('trplGUI:Sel:NoFigGUIHandle','No figGUI handle provided. Attempting to use callback figure');
				
				%% Try to get limits
				meas=getappdata(gcbf,'meas');
				
				%% Set coordinates top-left corner
				pt1=[max(meas.wl),min(meas.t)];
				pt2=[max(meas.wl),min(meas.t)];
				
				%% Set figGUI handle
				h_figGUI=gcbf;
				
			end
			
			%% Set figGUI handle
			if(isempty(h_figGUI) || ~ishandle(h_figGUI))
				throw(MException('trplGUI:Sel:BadFigGUIHandle','Invalid figGUI handle'));
			end
			aSel.h.figgui=h_figGUI;
							
			%% Get figGUI handles structure
			handles=guidata(aSel.h.figgui);

			%% Set default axes handles							% N.b. Cannot set these defaults in properties and require figGUI handles structure
			aSel.h.axes.crosshairs=handles.axMain;
			aSel.h.axes.spec=handles.axProfSpec;
			aSel.h.axes.kin=handles.axProfKin;

			%% Set default edit text handles
			aSel.h.etxt.wl1=handles.etxtSelWl1;
			aSel.h.etxt.wl2=handles.etxtSelWl2;
			aSel.h.etxt.t1=handles.etxtSelT1;
			aSel.h.etxt.t2=handles.etxtSelT2;

			%% Parse optional parameters
			if(~isempty(varargin))
				[names,values]=varargin2pairs(varargin);
				for ind=1:length(names)
					switch(lower(names{ind}))
						case {'axescrosshairs','axesspectrum','axeskinetic','etxtwl1','etxtwl2','etxtt1','etxtt2'}
							aSel.setHandles(names{ind},values{ind});
						case {'etxtformat','nearpix','linestyle','linewidth','linecolor','shadecolor'};
							aSel.setOpts(names{ind},values{ind});
						otherwise
							throw(MException('trplGUI:Sel:BadArg','Invalid constructor. Expected valid handle name or option name. Received %s',names{ind}));
					end
				end
			end
			
			%% Set inital and final selection points
			aSel.wl1=pt1(1);
			aSel.t1=pt1(2);
			aSel.wl2=pt2(1);
			aSel.t2=pt2(2);
			
		end
		
		%% Destructor
		function delete(aSel)
			
			%% Clear crosshairs
			delete(aSel.hs.crosshairs(~isempty(aSel.hs.crosshairs) & ishandle(aSel.hs.crosshairs)));
			
			%% Clear profiles
			if(~isempty(aSel.h.axes.spec) && ishandle(aSel.h.axes.spec))
				cla(aSel.h.axes.spec);
			end
			if(~isempty(aSel.h.axes.kin) && ishandle(aSel.h.axes.kin))
				cla(aSel.h.axes.kin);
			end
			
			%% Reset edit text boxes
			coords={'wl1','wl2','t1','t2'};
			for ind_coord=1:length(coords)
				if(~isempty(aSel.h.etxt.(coords{ind_coord})) && ishandle((aSel.h.etxt.(coords{ind_coord}))))
					set(aSel.h.etxt.(coords{ind_coord}),'String',coords{ind_coord});
				end
			end
			
		end
		
		%% Update
		function update(aSel)
			if(~isempty(aSel.wl1) && ~isempty(aSel.wl2) && ~isempty(aSel.t1) && ~isempty(aSel.t2))		% Catches initial property sets from constructor
				aSel.plotCH();							% Check for empty handles in these functions
				aSel.plotSpec();
				aSel.plotKin();
				aSel.setETxt();
			end
		end
		
		%% Assert													% Is this method really useful? If so, should check all properties
		%{
		function tf=assertSel(aSel,varargin)
			
			%% Check coordinate values
			tf.wl=(aSel.assertCoord(aSel.wl1,'wl') && aSel.assertCoord(aSel.wl2,'wl'));
			tf.t=(aSel.assertCoord(aSel.t1,'t') && aSel.assertCoord(aSel.t2,'t'));
			
			%% Check handles
			tf.h=ishandle(aSel.hs);													% Needs rewriting
			
			%% Parse varargin
			if(isempty(varargin))
				cond=(all(tf.wl))&(all(tf.t));
			else
				switch(lower(varargin{1}))
					case 'all'
						cond=(all(tf.wl))&(all(tf.t))&(all(tf.h));
					otherwise
						throw(MException('trplGUI:Sel:BadArg','Unrecognised optional argument. Expected "all". Received "%s"',varargin{1}));
				end
			end
			
			%% Set return value
			if(cond)
				tf=true;
			else
				tf=false;
			end
			
		end
		%}
		
		%% Proximity check
		function mask=isNear(aSel,pt)
			
			%% Get axes properties
			ax.lims=struct('wl',get(aSel.h.axes.crosshairs,'XLim'),'t',get(aSel.h.axes.crosshairs,'YLim'));
			ax.units=get(aSel.h.axes.crosshairs,'Units');
			set(aSel.h.axes.crosshairs,'Units','pixels');
			axpos=get(aSel.h.axes.crosshairs,'Position');
			set(aSel.h.axes.crosshairs,'Units',ax.units);
			ax.dims=struct('wl',axpos(3),'t',axpos(4));
			
			%% Define proximity
			near.pix=aSel.opts.nearpix;
			near.units=struct('wl',near.pix*abs(diff(ax.lims.wl))/ax.dims.wl,'t',near.pix*abs(diff(ax.lims.t))/ax.dims.t);
			
			%% Define default return values
			mask=[false,false;false,false];				% mask=[wl1,t1;wl2,t2]
			
			%% Check proximity
			if(pt(1)>=(aSel.wl1-near.units.wl) && pt(1)<=(aSel.wl1+near.units.wl))
				mask(1)=true;
			end
			if(pt(1)>=(aSel.wl2-near.units.wl) && pt(1)<=(aSel.wl2+near.units.wl))
				mask(2)=true;
			end
			if(pt(2)>=(aSel.t1-near.units.t) && pt(2)<=(aSel.t1+near.units.t))
				mask(3)=true;
			end
			if(pt(2)>=(aSel.t2-near.units.t) && pt(2)<=(aSel.t2+near.units.t))
				mask(4)=true;
			end
			
		end
		
		%% Assert coordinate value
		function [tf,bound]=assertCoord(aSel,coord,axis)
			
			%% Validate axis
			switch(lower(axis))
				case {'wl','t'}
				otherwise
					throw(MException('trplGUI:Sel:BadAxis','Axis not recognised. Expected "wl" or "t". Receieved "%s"',axis));
			end
			axis=lower(axis);
			
			%% Catch vector values
			if(~isscalar(coord))
				throw(MException('trplGUI:Sel:VecCoord','Selection coordinate must be scalar. Attempted to set coordinate to a vector of length %d',length(coord)));
			end
			
			%% Catch non-numeric coordinates
			if(~isnumeric(coord))
				throw(MException('trplGUI:Sel:NonNumCoord','Selection coordinate must be numeric. Attempted to set coordinate to a non-numeric value'));
			end
			
			%% Catch NaN and Inf coordinates
			if(any(~isfinite(coord)))
				throw(MException('trplGUI:Sel:NonFiniteCoord','Selection coordinate must be finite. Attempted to set coordinate to NaN or Inf'));
			end
			
			%% Check coordinates within measurements limits
			if(coord<aSel.lims.(axis)(1))
				tf=false;
				bound=aSel.lims.(axis)(1);
			elseif(coord>aSel.lims.(axis)(2))
				tf=false;
				bound=aSel.lims.(axis)(2);
			else
				tf=true;
				bound=[];
			end
			
		end
		
		%% Set options
		function setOpts(aSel,varargin)
			
			%% Parse varargin
			[opts,values]=varargin2pairs(varargin);																									%#ok<PROP>
			for ind=1:length(opts)																													%#ok<PROP>
				switch(lower(opts{ind}))																											%#ok<PROP>
					case 'nearpix'
						if(isnumeric(values{ind}))
							aSel.opts.nearpix=values{ind};
						else
							warning('trplGUI:Sel:BadNearPix','Non-numeric value of NearPix option. Setting to default value (%d)',aSel.opts.nearpix);
						end
					case 'etxtformat'
						aSel.opts.etxtformat=values{ind};
					case 'linestyle'
						aSel.opts.line.style=values{ind};
					case 'linewidth'
						aSel.opts.line.width=values{ind};
					case 'linecolor'
						aSel.opts.line.color=values{ind};
					case 'shadecolor'
						aSel.opts.shadecolor=values{ind};
					otherwise
						warning('trplGUI:Sel:BadOptionName','Invalid option. Expected "NearPix", "Marker", "MarkerEdgeColor", "MarkerFaceColor", "MarkerSize", "ETxtFormat". Received %s. Ignoring OptionName-OptionValue pair',opts{ind}); %#ok<PROP>
				end
			end
			
			%% Update
			aSel.update();
			
		end
		
		%% Set handles
		function setHandles(aSel,varargin)
			
			%% Parse varargin
			[names,values]=varargin2pairs(varargin);
			names=lower(names);
			for ind=1:length(names)
				switch(names{ind})
					case 'axescrosshairs'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aSel.h.axes.crosshairs=values{ind};
						else
							warning('trplGUI:Sel:BadCrosshairAxesHandle','Invalid axes handle for crosshairs. Setting h.axes.crosshairs to []');
							aSel.h.axes.crosshairs=[];
						end
					case 'axesspectrum'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aSel.h.axes.spec=values{ind};
						else
							warning('trplGUI:Sel:BadSpectrumAxesHandle','Invalid axes handle for spectral profile. Setting h.axes.spec to []');
							aSel.h.axes.spec=[];
						end
					case 'axeskinetic'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aSel.h.axes.kin=values{ind};
						else
							warning('trplGUI:Sel:BadKineticAxesHandle','Invalid axes handle for kinetic profile. Setting h.axes.kin to []');
							aSel.h.axes.kin=[];
						end
					case 'etxtwl1'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aSel.h.etxt.wl1=values{ind};
						else
							warning('trplGUI:Sel:BadWl1EtxtHandle','Invalid edit text handle for wl1. Setting h.etxt.wl1 to []');
							aSel.h.etxt.wl1=[];
						end
					case 'etxtwl2'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aSel.h.etxt.wl2=values{ind};
						else
							warning('trplGUI:Sel:BadWl2EtxtHandle','Invalid edit text handle for wl2. Setting h.etxt.wl2 to []');
							aSel.h.etxt.wl2=[];
						end
					case 'etxtt1'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aSel.h.etxt.t1=values{ind};
						else
							warning('trplGUI:Sel:BadT1EtxtHandle','Invalid edit text handle for t1. Setting h.etxt.t1 to []');
							aSel.h.etxt.t1=[];
						end
					case 'etxtt2'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aSel.h.etxt.t2=values{ind};
						else
							warning('trplGUI:Sel:BadT2EtxtHandle','Invalid edit text handle for t2. Setting h.etxt.t2 to []');
							aSel.h.etxt.t2=[];
						end
					otherwise
						warning('trplGUI:Sel:BadHandleName','Invalid handle name. Expected "AxesCrosshairs", "AxesSpectrum", "AxesKinetic", "ETxtWl1", "ETxtWl2", "ETxtT1", or "ETxtT2". Received %s. Ignoring HandleName-HandleValue pair',names{ind});
				end
			end
			
			%% Update
			aSel.update();
			
		end
		
	end
	
	%% Private %%
	methods
		
		%% Plot crosshairs
		function plotCH(aSel)
			
			if(~isempty(aSel.h.axes.crosshairs) && ishandle(aSel.h.axes.crosshairs))
				
				%% Plot new crosshairs
				if(any(isempty(aSel.hs.crosshairs)) || any(~ishandle(aSel.hs.crosshairs)))
					
					%% Plot lines
					aSel.h.crosshairs.wl1=line('Parent',aSel.h.axes.crosshairs,'XData',[aSel.wl1,aSel.wl1],'YData',aSel.lims.t,'Color',aSel.opts.line.color,'LineStyle',aSel.opts.line.style,'LineWidth',aSel.opts.line.width);
					aSel.h.crosshairs.wl2=line('Parent',aSel.h.axes.crosshairs,'XData',[aSel.wl2,aSel.wl2],'YData',aSel.lims.t,'Color',aSel.opts.line.color,'LineStyle',aSel.opts.line.style,'LineWidth',aSel.opts.line.width);
					aSel.h.crosshairs.t1=line('Parent',aSel.h.axes.crosshairs,'XData',aSel.lims.wl,'YData',[aSel.t1,aSel.t1],'Color',aSel.opts.line.color,'LineStyle',aSel.opts.line.style,'LineWidth',aSel.opts.line.width);
					aSel.h.crosshairs.t2=line('Parent',aSel.h.axes.crosshairs,'XData',aSel.lims.wl,'YData',[aSel.t2,aSel.t2],'Color',aSel.opts.line.color,'LineStyle',aSel.opts.line.style,'LineWidth',aSel.opts.line.width);
					
					%% Plot crossover points
					aSel.opts.pt.color=hsv2rgb(rgb2hsv(get(aSel.h.crosshairs.wl1,'Color')).*[1,1,0.25]);			% Use get to convert string colorspecs
					aSel.h.crosshairs.pt11=line('Parent',aSel.h.axes.crosshairs,'XData',aSel.wl1,'YData',aSel.t1,'Color',aSel.opts.pt.color,'LineStyle',aSel.opts.pt.linestyle,'Marker',aSel.opts.pt.marker,'MarkerSize',aSel.opts.pt.size);
					aSel.h.crosshairs.pt12=line('Parent',aSel.h.axes.crosshairs,'XData',aSel.wl1,'YData',aSel.t2,'Color',aSel.opts.pt.color,'LineStyle',aSel.opts.pt.linestyle,'Marker',aSel.opts.pt.marker,'MarkerSize',aSel.opts.pt.size);
					aSel.h.crosshairs.pt21=line('Parent',aSel.h.axes.crosshairs,'XData',aSel.wl2,'YData',aSel.t1,'Color',aSel.opts.pt.color,'LineStyle',aSel.opts.pt.linestyle,'Marker',aSel.opts.pt.marker,'MarkerSize',aSel.opts.pt.size);
					aSel.h.crosshairs.pt22=line('Parent',aSel.h.axes.crosshairs,'XData',aSel.wl2,'YData',aSel.t2,'Color',aSel.opts.pt.color,'LineStyle',aSel.opts.pt.linestyle,'Marker',aSel.opts.pt.marker,'MarkerSize',aSel.opts.pt.size);
					
					%% Set line pointers
					iptSetPointerBehavior(aSel.h.crosshairs.wl1,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.line.pointer.wl));
					iptSetPointerBehavior(aSel.h.crosshairs.wl2,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.line.pointer.wl));
					iptSetPointerBehavior(aSel.h.crosshairs.t1,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.line.pointer.t));
					iptSetPointerBehavior(aSel.h.crosshairs.t2,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.line.pointer.t));
					
					%% Set crossover point pointers
					iptSetPointerBehavior(aSel.h.crosshairs.pt11,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.pt.pointer));
					iptSetPointerBehavior(aSel.h.crosshairs.pt12,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.pt.pointer));
					iptSetPointerBehavior(aSel.h.crosshairs.pt21,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.pt.pointer));
					iptSetPointerBehavior(aSel.h.crosshairs.pt22,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.pt.pointer));
					
					%% Enable pointer manager
					iptPointerManager(get(aSel.h.axes.crosshairs,'Parent'),'enable');
					
					
					
					
				%% Modify existing crosshairs
				else
					
					%% Modify lines
					set(aSel.h.crosshairs.wl1,'Parent',aSel.h.axes.crosshairs,'XData',[aSel.wl1,aSel.wl1],'YData',aSel.lims.t,'Color',aSel.opts.line.color,'LineStyle',aSel.opts.line.style,'LineWidth',aSel.opts.line.width);
					set(aSel.h.crosshairs.wl2,'Parent',aSel.h.axes.crosshairs,'XData',[aSel.wl2,aSel.wl2],'YData',aSel.lims.t,'Color',aSel.opts.line.color,'LineStyle',aSel.opts.line.style,'LineWidth',aSel.opts.line.width);
					set(aSel.h.crosshairs.t1,'Parent',aSel.h.axes.crosshairs,'XData',aSel.lims.wl,'YData',[aSel.t1,aSel.t1],'Color',aSel.opts.line.color,'LineStyle',aSel.opts.line.style,'LineWidth',aSel.opts.line.width);
					set(aSel.h.crosshairs.t2,'Parent',aSel.h.axes.crosshairs,'XData',aSel.lims.wl,'YData',[aSel.t2,aSel.t2],'Color',aSel.opts.line.color,'LineStyle',aSel.opts.line.style,'LineWidth',aSel.opts.line.width);

					%% Modify crossover points
					set(aSel.h.crosshairs.pt11,'Parent',aSel.h.axes.crosshairs,'XData',aSel.wl1,'YData',aSel.t1,'Color',aSel.opts.pt.color,'LineStyle',aSel.opts.pt.linestyle,'Marker',aSel.opts.pt.marker,'MarkerSize',aSel.opts.pt.size);
					set(aSel.h.crosshairs.pt12,'Parent',aSel.h.axes.crosshairs,'XData',aSel.wl1,'YData',aSel.t2,'Color',aSel.opts.pt.color,'LineStyle',aSel.opts.pt.linestyle,'Marker',aSel.opts.pt.marker,'MarkerSize',aSel.opts.pt.size);
					set(aSel.h.crosshairs.pt21,'Parent',aSel.h.axes.crosshairs,'XData',aSel.wl2,'YData',aSel.t1,'Color',aSel.opts.pt.color,'LineStyle',aSel.opts.pt.linestyle,'Marker',aSel.opts.pt.marker,'MarkerSize',aSel.opts.pt.size);
					set(aSel.h.crosshairs.pt22,'Parent',aSel.h.axes.crosshairs,'XData',aSel.wl2,'YData',aSel.t2,'Color',aSel.opts.pt.color,'LineStyle',aSel.opts.pt.linestyle,'Marker',aSel.opts.pt.marker,'MarkerSize',aSel.opts.pt.size);

					%% Set line pointers							% These are a bit heavy and unnecessary
					%{
					iptSetPointerBehavior(aSel.h.crosshairs.wl1,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.line.pointer.wl));
					iptSetPointerBehavior(aSel.h.crosshairs.wl2,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.line.pointer.wl));
					iptSetPointerBehavior(aSel.h.crosshairs.t1,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.line.pointer.t));
					iptSetPointerBehavior(aSel.h.crosshairs.t2,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.line.pointer.t));
					%}

					%% Set crossover point pointers
					%{
					iptSetPointerBehavior(aSel.h.crosshairs.pt11,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.pt.pointer));
					iptSetPointerBehavior(aSel.h.crosshairs.pt12,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.pt.pointer));
					iptSetPointerBehavior(aSel.h.crosshairs.pt21,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.pt.pointer));
					iptSetPointerBehavior(aSel.h.crosshairs.pt22,@(figHandle,currentPoint) setptr(figHandle,aSel.opts.pt.pointer));
					%}

				end
				
			end
			
		end	
		
		%% Plot spectral profile
		function plotSpec(aSel)
			
			if(~isempty(aSel.h.axes.spec) && ishandle(aSel.h.axes.spec))
				
				%% Get figGUI appdata
				meas=getappdata(aSel.h.figgui,'meas');
				
				%% Plot profile
				plot(aSel.h.axes.spec,meas.wl',trplGetSpec(meas.data,meas.t,aSel.ts));
				
				%% Shade selection
				shade.color=aSel.opts.shadecolor;
				axProf.YLim=get(aSel.h.axes.spec,'YLim');
				if(aSel.wl_span>0)
					shade.position=[min(aSel.wls),axProf.YLim(1),aSel.wl_span,diff(axProf.YLim)];
					rectangle('Parent',aSel.h.axes.spec,'Position',shade.position,'LineStyle','none','FaceColor',shade.color);
				else
					shade.width=get(aSel.h.crosshairs.wl1,'LineWidth');
					line('Parent',aSel.h.axes.spec,'XData',[aSel.wl1,aSel.wl1],'YData',axProf.YLim,'Color',shade.color,'LineWidth',shade.width);
					%{
					axProf.units=get(handles.axProfSpec,'Units');												% Generalise this into a unit2data function?
					set(handles.axProfSpec,'Units','points');
					axProf.width.points=get(handles.axProfSpec,'Position');
					axProf.width.points=axProf.width.points(3);
					set(handles.axProfSpec,'Units',axProf.units);
					axProf.width.data=diff(get(handles.axProfSpec,'XLim'));
					halfwidth=0.5*get(aSel.h.wl1,'LineWidth')*axProf.width.data/axProf.width.points;
					rect.position=[min(aSel.wls)-halfwidth,axProf.YLim(1),2*halfwidth,diff(axProf.YLim)];
					%}
				end
				set(aSel.h.axes.spec,'Children',flipud(get(aSel.h.axes.spec,'Children')));

			end
						
		end
		
		%% Plot kinetic profile
		function plotKin(aSel)
			
			
			if(~isempty(aSel.h.axes.kin) && ishandle(aSel.h.axes.kin))
				
				%% Get figGUI appdata
				meas=getappdata(aSel.h.figgui,'meas');
				
				%% Plot profile
				plot(aSel.h.axes.kin,trplGetKin(meas.data,meas.wl,aSel.wls),meas.t);
				
				%% Shade selection
				shade.color=aSel.opts.shadecolor;
				axProf.XLim=get(aSel.h.axes.kin,'XLim');
				if(aSel.t_span>0)
					shade.position=[axProf.XLim(1),min(aSel.ts),diff(axProf.XLim),aSel.t_span];
					rectangle('Parent',aSel.h.axes.kin,'Position',shade.position,'LineStyle','none','FaceColor',shade.color);
				else
					shade.width=get(aSel.h.crosshairs.t1,'LineWidth');
					line('Parent',aSel.h.axes.kin,'XData',axProf.XLim,'YData',[aSel.t1,aSel.t1],'Color',shade.color,'LineWidth',shade.width);
				end
				set(aSel.h.axes.kin,'Children',flipud(get(aSel.h.axes.kin,'Children')));
				
			end
			
		end
		
		%% Set edit text boxes
		function setETxt(aSel)
			
			%% Set etxt boxes
			if(~isempty(aSel.h.etxt.wl1) && ishandle(aSel.h.etxt.wl1))
				set(aSel.h.etxt.wl1,'String',num2str(aSel.wl1,aSel.opts.etxtformat));
			end
			if(~isempty(aSel.h.etxt.wl2) && ishandle(aSel.h.etxt.wl2))
				set(aSel.h.etxt.wl2,'String',num2str(aSel.wl2,aSel.opts.etxtformat));
			end
			if(~isempty(aSel.h.etxt.t1) && ishandle(aSel.h.etxt.t1))
				set(aSel.h.etxt.t1,'String',num2str(aSel.t1,aSel.opts.etxtformat));
			end
			if(~isempty(aSel.h.etxt.t2) && ishandle(aSel.h.etxt.t2))
				set(aSel.h.etxt.t2,'String',num2str(aSel.t2,aSel.opts.etxtformat));
			end
		
		end
		
	end
	
	%% Access %%
	methods	
		
		%% Set: wl1
		function aSel=set.wl1(aSel,wl)
			
			%% Validate value
			[ret,bound]=aSel.assertCoord(wl,'wl');
			if(~ret)
				warning('trplGUI:Sel:BadWl1','Wavelength value outside measurement bound. Expected values between %g and %g. Received %g. Setting wl1 to bound (%g)',aSel.lims.wl(1),aSel.lims.wl(2),wl,bound); %#ok<MCSUP>
				wl=bound;
			end
			
			%% Set value
			aSel.wl1=wl;
			aSel.update();
			
		end
		
		%% Set: wl2
		function aSel=set.wl2(aSel,wl)
			
			%% Validate value
			[ret,bound]=aSel.assertCoord(wl,'wl');
			if(~ret)
				warning('trplGUI:Sel:BadWl2','Wavelength value outside measurement bound. Expected values between %g and %g. Received %g. Setting wl2 to bound (%g)',aSel.lims.wl(1),aSel.lims.wl(2),wl,bound); %#ok<MCSUP>
				wl=bound;
			end
			
			%% Set value
			aSel.wl2=wl;
			aSel.update();
			
		end
		
		%% Set: t1
		function aSel=set.t1(aSel,t)
				
			%% Validate value
			[ret,bound]=aSel.assertCoord(t,'t');
			if(~ret)
				warning('trplGUI:Sel:BadT1','Time value outside measurement bound. Expected values between %g and %g. Received %g. Setting t1 to bound (%g)',aSel.lims.t(1),aSel.lims.t(2),t,bound); %#ok<MCSUP>
				t=bound;
			end
			
			%% Set value
			aSel.t1=t;
			aSel.update();
			
		end
		
		%% Set: t2
		function aSel=set.t2(aSel,t)
			
			%% Validate value
			[ret,bound]=aSel.assertCoord(t,'t');
			if(~ret)
				warning('trplGUI:Sel:BadT2','Time value outside measurement bound. Expected values between %g and %g. Received %g. Setting t2 to bound (%g)',aSel.lims.t(1),aSel.lims.t(2),t,bound); %#ok<MCSUP>
				t=bound;
			end
			
			%% Set value
			aSel.t2=t;
			aSel.update();
		
		end
				
		%% Get: wls
		function wls=get.wls(aSel)
			wls=[aSel.wl1,aSel.wl2];
		end
		
		%% Get wl_span
		function wl_span=get.wl_span(aSel)
			wl_span=abs(diff(aSel.wls));
		end
		
		%% Get: ts
		function ts=get.ts(aSel)
			ts=[aSel.t1,aSel.t2];
		end
		
		%% Get t_span
		function t_span=get.t_span(aSel)
			t_span=abs(diff(aSel.ts));
		end
		
		%% Get: hs
		function hs=get.hs(aSel)
			hs=struct('crosshairs',[aSel.h.crosshairs.wl1;aSel.h.crosshairs.wl2;aSel.h.crosshairs.t1;aSel.h.crosshairs.t2;aSel.h.crosshairs.pt11;aSel.h.crosshairs.pt12;aSel.h.crosshairs.pt21;aSel.h.crosshairs.pt22],'etxt',[aSel.h.etxt.wl1;aSel.h.etxt.wl2;aSel.h.etxt.t1;aSel.h.etxt.t2]);
		end
		
		%% Get: lims
		function lims=get.lims(aSel)
			meas=getappdata(aSel.h.figgui,'meas');
			lims=struct('wl',[min(meas.wl),max(meas.wl)],'t',[min(meas.t),max(meas.t)]);
		end
		
	end
		
end