%% Name:		varargin2pairs.m
%% Description:	Parse the contents of a name-value pair cell array into two seperate cell arrays
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as 
%%				published by the Free Software Foundation, either version 3 of the License, or any later version. This program is distributed
%%				in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
%%				FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU
%%				General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

function [names,values]=varargin2pairs(a_varargin)

	%% Syntax:
	%%
	%%		[names,values]=varargin2pairs(a_varargin)
	%%
	%% Description:
	%%
	%%		[names,values]=varargin2pairs(a_varargin) reformats the contents of a cell array containing name-value pairs (e.g. varargin) into
	%%		separate cell arrays, names and values, such that names{i} corresponds to values{i}.
	%%
	%%			Arguments:
	%%				a_varargin:	A cell array containing name-value pairs
	%%
	%%			Returns:
	%%				names:		A cell array of the names supplied
	%%				values:		A cell array of the corresponding values
	%%

	%% Check varargin is cell array
	if(~iscell(a_varargin))
		throw(MException('varargin2pairs:NonCell','Argument is not a cell array. Expected a cell array with an even number of elements'));
	end

	%% Check property-value pairing
	if(mod(length(a_varargin),2)~=0)
		throw(MException('varargin2pairs:OddPairs','Odd PropertyName-PropertyValue pairing. Expected even number of arguments. Received %d arguments)',length(a_varargin)));
	end

	%% Parse varargin
	names=a_varargin(1:2:(length(a_varargin)-1));
	values=a_varargin(2:2:length(a_varargin));
	
	%% Check names are all strings
	if(~iscellstr(names))
		throw(MException('varargin2pairs:BadNames','One or more PropertyNames are not strings'));
	end

end