%% Name:		trplGUI.m
%% Description:	A graphical user interface for analysis of Time Resolved Photoluminescence data taken in the Chameleon Lab.
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.
%%
%%				trplGUI makes use of findjobj, Copyright  2010 Yair M. Altman [altmany(at)gmail.com].

function varargout = trplGUI(varargin)
% TRPLGUI M-file for trplGUI.fig
%      TRPLGUI, by itself, creates a new TRPLGUI or raises the existing
%      singleton*.
%
%      H = TRPLGUI returns the handle to a new TRPLGUI or the handle to
%      the existing singleton*.
%
%      TRPLGUI('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in TRPLGUI.M with the given input arguments.
%
%      TRPLGUI('Property','Value',...) creates a new TRPLGUI or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before trplGUI_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to trplGUI_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help trplGUI

% Last Modified by GUIDE v2.5 16-Sep-2012 18:29:27

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @trplGUI_OpeningFcn, ...
                   'gui_OutputFcn',  @trplGUI_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT
end

% --- Executes just before trplGUI is made visible.
function trplGUI_OpeningFcn(hObject, eventdata, handles, varargin)																				%#ok<*INUSL>
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to trplGUI (see VARARGIN)

	%% Set command line output for trplGUI
	handles.output = hObject;

	%% Add common directory to path if not already present
	if(~isdeployed() && isempty(strfind(path,strrep(which('trplGUI'),'trplGUI.m','common'))))
		addpath(strrep(which('trplGUI'),'trplGUI.m','common'));
		state.comm_add=true;
	else
		state.comm_add=false;
	end
	
	%% Initialise state appdata
	state.dirty=false;
	
	%% Store state appdata
	setappdata(handles.figGUI,'state',state);
	
	
	
	
	%% Initialise selections and corrections appdata
	setappdata(handles.figGUI,'sels',[]);
	setappdata(handles.figGUI,'corrs',struct('t0',[],'curv',[]));
	
	%% Set selection edit text boxes to highlight selection on focus
	set(handles.figGUI,'Visible','on');																	% Cannot use findjobj until figGUI is visible
	h_etxts=findjobj(handles.figGUI,'class','edit');
	set(h_etxts,'FocusGainedCallback',@trplGUI_etxt_FocusGainedCallback);
	
	%% Disable selection bound warnings
	warning('off','trplGUI:Sel:BadWl1');
	warning('off','trplGUI:Sel:BadWl2');
	warning('off','trplGUI:Sel:BadT1');
	warning('off','trplGUI:Sel:BadT2');
	
	% Update handles
	guidata(hObject,handles);
	
	
	
	%% Link axes
	links.axX=linkprop([handles.axMain,handles.axProfSpec],{'XLim','XDir'});
	links.axY=linkprop([handles.axMain,handles.axProfKin],{'YLim','YDir'});
	setappdata(handles.figGUI,'links',links);
	
	%% Set default colormap
	colormap(handles.axMain,'hot');
% 	cptcmap('cool-warm');
% 	set(handles.figGUI,'Colormap',pmkmp(64,'LinearL'));
% 	set(handles.figGUI,'Colormap',pmkmp(64,'CubicYF'));
	
	%% Handle input arguments
	if(~isempty(varargin))
		
		try
			[params,values]=varargin2pairs(varargin);
			ind_file=strcmp('file',params);																	% Check for more arguments here
		catch ME
			warning('trplGUI:Opening:BadArgs','Invalid arguments. Starting default trplGUI session');
			ind_file=false;
		end
		
		if(any(ind_file))
			trplGUI_figGUI_Load(handles.figGUI,[],values{ind_file});
		end
		
		
	end
	
	%% Update handles
	guidata(hObject,handles);
	

end

% --- Outputs from this function are returned to the command line.
function varargout = trplGUI_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

	%% Check for updates
	ver_current=trplGUI_figGUI_Current;
	ver_latest=trplGUI_figGUI_Latest();
	if(ver_latest(1)>ver_current(1) || (ver_latest(1)==ver_current(1) && ver_latest(2)>ver_current(2)) || (ver_latest(1)==ver_current(1) && ver_latest(2)==ver_current(2) && ver_latest(3)>ver_current(3)))
		
		handles_updates.figUpdates=msgbox({'A new version of trplGUI is available:';[];sprintf('   Current version: v%d.%d.%d',ver_current);sprintf('   Latest version: v%d.%d.%d',ver_latest);[];[];[];[]},'trplGUI - Update Notification','help','modal');
		pbtUpdates.Parent=handles_updates.figUpdates;
		pbtUpdates.HRef='http://code.google.com/p/ma-code/downloads/list';
		pbtUpdates.String=sprintf('<html><a href="%s">Download trplGUI v%d.%d.%d</a></html>',pbtUpdates.HRef,ver_latest(1),ver_latest(2),ver_latest(3));
		pbtUpdates.BackgroundColor=get(0,'defaultUicontrolBackgroundColor');
		pbtUpdates.Units='Normalized';
		pbtUpdates.Position=[0.25,0.35,0.6,0.12];
		handles_updates.pbtUpdates=trplGUI_figGUI_CreateJButton(pbtUpdates);
		
	end

	% Get default command line output from handles structure
	varargout{1} = handles.output;
	
end




%%-- trplGUI_figGUI* --%%

function trplGUI_figGUI_Close(hObject, eventdata, handles)																						%#ok<*DEFNU>

	%% Warn on previous t0 correction
	if(trplGUI_figGUI_Dirty(handles.figGUI))
		ret=questdlg('Save changes to measurement data?','trplGUI - Save changes?','Yes','No','Yes');
		if(strcmp(ret,'Yes'))
			meas=getappdata(gcbf,'meas');
			file_path=fileparts(meas.path);
			trplSave(meas.data,meas.wl,meas.t,meas.stat,file_path);
		elseif(isempty(ret))
			return
		end
	end

	%% Get appdata
	state=getappdata(hObject,'state');
	sels=getappdata(hObject,'sels');

	%% Delete selections
	if(~isempty(sels))
		delete(sels);
	end
	
	%% Build figure close array
	h_close=[];
	if(isfield(handles,'figCorrCurv'))
		h_close=[h_close;handles.figCorrCurv];
	end
	if(isfield(handles,'figCorrT0'))
		h_close=[h_close;handles.figCorrT0];
	end
	if(isfield(handles,'figCorrSubRef'))
		h_close=[h_close;handles.figCorrSubRef];
	end
	h_close=[h_close;hObject];
	
	%% Close main trplGUI figure
	delete(h_close);
	
	%% Remove common directory from path if added in opening function
	if(state.comm_add)
		rmpath(strrep(which('trplGUI'),'trplGUI.m','common'));
	end

end

function trplGUI_figGUI_WindowButtonDownSel_Callback(hObject, eventdata, handles)

	%% Return if no measurement data loaded
	if (~isappdata(gcbf,'meas'))
		return
	end

	%% Return if not left-click
	if(~strcmp(get(gcbf,'SelectionType'),'normal'))
		return
	end
	
	%% Get axes limits
	lims.wl=get(handles.axMain,'XLim');
	lims.t=get(handles.axMain,'YLim');
	
	%% Get coordinates of mouse button down
	pt1=get(handles.axMain,'CurrentPoint');
	pt1=pt1(1,1:2);
	
	%% Return if mouse coordinates outside axes
	if(pt1(1)<lims.wl(1) || pt1(1)>lims.wl(2) || pt1(2)<lims.t(1) || pt1(2)>lims.t(2))
		return
	end
	
	%% Get selections appdata
	sels=getappdata(gcbf,'sels');
	
	%% No previous selection exists
	if(isempty(sels))
		
		%% Create new selection
		sels=trplGUIClasses.Sel(pt1,pt1,gcbf);
		mod_mask=[false,false;true,true];					% mod_mask=[wl1,t1;wl2,t2]
		
	%% Previous selection exists
	else
		
		%% Check if click is near previous selection
		mod_mask=sels.isNear(pt1);
		if(~any(mod_mask))
			
			%% Not near - create new selection
			sels.delete();
			sels=trplGUIClasses.Sel(pt1,pt1,gcbf);
			mod_mask=[false,false;true,true];				% mod_mask=[wl1,t1;wl2,t2]
			
		end
		
	end
	
	%% Update selections appdata
	setappdata(gcbf,'sels',sels);
	
	%% Start tracking mouse motion
	set(hObject,'WindowButtonMotionFcn',{@trplGUI_figGUI_WindowButtonMotionSel_Callback,mod_mask});

end

function trplGUI_figGUI_WindowButtonDownT0_Callback(hObject, eventdata)																			%#ok<*INUSD>

	%% Get handles
	handles=guidata(hObject);

	%% Return if not left-click
	if(~strcmp(get(gcbf,'SelectionType'),'normal'))
		return
	end
	
	%% Get axes limits
	lims.wl=get(handles.axMain,'XLim');
	lims.t=get(handles.axMain,'YLim');
	
	%% Get coordinates of mouse button down
	click=get(handles.axMain,'CurrentPoint');
	click=click(1,1:2);
	
	%% Return if mouse coordinates outside axes
	if(click(1)<lims.wl(1) || click(1)>lims.wl(2) || click(2)<lims.t(1) || click(2)>lims.t(2))
		return
	end
	
	%% Get corrections appdata
	corrs=getappdata(gcbf,'corrs');
	
	%% Check if click is near t0 points
	pts.number=length(corrs.t0.pts);
	pts.mask=false(1,pts.number);
	for ind_pt=1:pts.number
		pts.mask(ind_pt)=corrs.t0.pts(ind_pt).isNear(click);
	end
	
	%% Near point
	if(any(pts.mask))
		set(hObject,'WindowButtonMotionFcn',{@trplGUI_figGUI_WindowButtonMotionT0_Callback,pts.mask});
		
	%% Near average
	elseif(corrs.t0.avg.isNear(click))
		set(hObject,'WindowButtonMotionFcn',{@trplGUI_figGUI_WindowButtonMotionT0_Callback});				% This is a bit ugly
	end

end

function trplGUI_figGUI_WindowButtonDownCurv_Callback(hObject, eventdata)

	%% Get handles
	handles=guidata(hObject);

	%% Return if not left-click
	if(~strcmp(get(gcbf,'SelectionType'),'normal'))
		return
	end
	
	%% Get axes limits
	lims.wl=get(handles.axMain,'XLim');
	lims.t=get(handles.axMain,'YLim');
	
	%% Get coordinates of mouse button down
	click=get(handles.axMain,'CurrentPoint');
	click=click(1,1:2);
	
	%% Return if mouse coordinates outside axes
	if(click(1)<lims.wl(1) || click(1)>lims.wl(2) || click(2)<lims.t(1) || click(2)>lims.t(2))
		return
	end
	
	%% Get corrections appdata
	corrs=getappdata(gcbf,'corrs');
	
	%% Check if click is near t0 points
	pts.number=length(corrs.curv.t0.pts);
	pts.mask=false(1,pts.number);
	for ind_pt=1:pts.number
		pts.mask(ind_pt)=corrs.curv.t0.pts(ind_pt).isNear(click);
	end
	
	%% Check if click is near parabola points
	parab_mask=corrs.curv.parab.isNear(click);
	
	%% Near t0 point
	if(any(pts.mask))
		set(hObject,'WindowButtonMotionFcn',{@trplGUI_figGUI_WindowButtonMotionCurv_Callback,'t',pts.mask});
	
	%% Near parabola
	elseif(parab_mask(1))
		
		%% Near parabola point
		if(any(parab_mask(2:4)))
			set(hObject,'WindowButtonMotionFcn',{@trplGUI_figGUI_WindowButtonMotionCurv_Callback,'p',find(parab_mask(2:4),1)});
			
		%% Near parabola line
		else
			set(hObject,'WindowButtonMotionFcn',{@trplGUI_figGUI_WindowButtonMotionCurv_Callback,'l'});
		end
		
	end

end

function trplGUI_figGUI_WindowButtonUp_Callback(hObject, eventdata, handles)

	%% Stop tracking mouse motion
	if(~isempty(get(hObject,'WindowButtonMotionFcn')))
		set(hObject,'WindowButtonMotionFcn',[]);
	end

end

function trplGUI_figGUI_Replot(hObject)

	%% Get handles and appdata
	handles=guidata(hObject);
	meas=getappdata(handles.figGUI,'meas');
	
	%% Plot new data on main axis
	trplPlot(handles.axMain,meas.data,meas.wl,meas.t);
	
	%% Add axis labels
	xlabel(handles.axMain,['Wavelength [',meas.stat.Scaling.ScalingXUnit,']']);
	ylabel(handles.axMain,['Time [',meas.stat.Scaling.ScalingYUnit,']']);
	xlabel(handles.axProfSpec,['Wavelength [',meas.stat.Scaling.ScalingXUnit,']']);									% Would be better to link, but doesn't seem to work
	ylabel(handles.axProfSpec,[meas.stat.Acquisition.ZAxisLabel,' [',lower(meas.stat.Acquisition.ZAxisUnit),'\cdot',meas.stat.Scaling.ScalingYUnit,']']);
	ylabel(handles.axProfKin,['Time [',meas.stat.Scaling.ScalingYUnit,']']);										% Would be better to link, but doesn't seem to work
	xlabel(handles.axProfKin,[meas.stat.Acquisition.ZAxisLabel,' [',lower(meas.stat.Acquisition.ZAxisUnit),'\cdot',meas.stat.Scaling.ScalingXUnit,']']);
	
	%% Plot colorbar
	set(handles.figGUI,'CurrentAxes',handles.axCB);
	data_lim=[min(min(meas.data)),max(max(meas.data))];
	image([0,1],fliplr(data_lim),[length(colormap):-1:1]');
	data_lim_quart=diff(data_lim)/4;
	data_quarts=data_lim(1)+[data_lim_quart,2*data_lim_quart,3*data_lim_quart];
	set(handles.axCB,'YDir','normal','XTick',[],'YTick',data_quarts,'YTickLabel',sprintf('%g|%g|%g',data_quarts),'Box','off');
	ylabel(handles.axCB,[meas.stat.Acquisition.ZAxisLabel,' [',lower(meas.stat.Acquisition.ZAxisUnit),']']);
	set(handles.stxtCBMin,'String',sprintf('(%g)',data_lim(1)));
	set(handles.stxtCBMax,'String',sprintf('(%g)',data_lim(2)));
	set(handles.etxtCBMin,'String',sprintf('%g',data_lim(1)));
	set(handles.etxtCBMax,'String',sprintf('%g',data_lim(2)));
	
	%% Associate context menus with axes' children
	set(get(handles.axMain,'Children'),'UIContextMenu',get(handles.axMain,'UIContextMenu'));
	set(get(handles.axCB,'Children'),'UIContextMenu',get(handles.axCB,'UIContextMenu'));
	
	%% Handle previous selections
	sels=getappdata(handles.figGUI,'sels');
	if(~isempty(sels))
		
		%% Selection within new measurement ranges - update
		if(sels.assertCoord(sels.wl1,'wl') && sels.assertCoord(sels.wl2,'wl') && sels.assertCoord(sels.t1,'t') && sels.assertCoord(sels.t2,'t'))
			sels.update();
		
		%% Selection outside new measurement ranges - clear
		else
			sels.delete();
			sels=[];
		end
		
		%% Set selections appdata
		setappdata(handles.figGUI,'sels',sels)
		
	end
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_figGUI_WindowButtonMotionSel_Callback(hObject,eventdata,mod_mask)

	%% Get handles and appdata
	handles=guidata(gcbf);
	sels=getappdata(gcbf,'sels');
		
	%% Get coordinates of current mouse position
	pt2=get(handles.axMain,'CurrentPoint');
	pt2=pt2(1,1:2);

	%% Set specified selection properties
	props={'wl1','t1';'wl2','t2'};
	props=props(mod_mask);
	values=[pt2;pt2];
	values=values(mod_mask);
	for ind=1:length(props)
		sels.(props{ind})=values(ind);
	end
	
	%% Update selections appdata
	setappdata(gcbf,'sels',sels);

end

function trplGUI_figGUI_WindowButtonMotionT0_Callback(hObject,eventdata,varargin)

	%% Get handles and corrections appdata
	handles=guidata(hObject);
	corrs=getappdata(hObject,'corrs');
	
	%% Get coordinates of current mouse position
	pt=get(handles.axMain,'CurrentPoint');
	pt=pt(1,1:2);
	
	%% No modification mask - move average t0 line
	if(isempty(varargin))
		set(corrs.t0.avg,'t1',pt(2),'t2',pt(2));
	
	%% Modification mask - move individual t0 points
	else
		
		%% Move specified t0 points and lines
		set(corrs.t0.lines(varargin{1}),'YData',[pt(2),pt(2)]);
		set(corrs.t0.pts(varargin{1}),'t',pt(2));
		
		%% Move average t0 line
		avg=mean([corrs.t0.pts.t]);
		set(corrs.t0.avg,'t1',avg,'t2',avg);
		
	end

end

function trplGUI_figGUI_WindowButtonMotionCurv_Callback(hObject,eventdata,clicked,varargin)

	%% Get handles and corrections appdata
	handles=guidata(hObject);
	corrs=getappdata(hObject,'corrs');
	
	%% Get coordinates of current mouse position
	pt=get(handles.axMain,'CurrentPoint');
	pt=pt(1,1:2);
	
	switch(clicked)
		
		%% Move t0 point
		case 't'

			%% Move specified t0 point
			set(corrs.curv.t0.lines(varargin{1}),'YData',[pt(2),pt(2)]);
			set(corrs.curv.t0.pts(varargin{1}),'t',pt(2));

			%% Refit parabola
			final.coeffs=[ones(length(corrs.curv.t0.pts),1),[corrs.curv.t0.pts.wl]',([corrs.curv.t0.pts.wl]').^2]\([corrs.curv.t0.pts.t]');		% mldivide lsq solution
			final.t=[ones(3,1),corrs.curv.parab.wls',(corrs.curv.parab.wls').^2]*final.coeffs;
			set(corrs.curv.parab,'t1',final.t(1),'t2',final.t(2),'t3',final.t(3));
		
			
		%% Move parabola point
		case 'p'
			set(corrs.curv.parab,sprintf('wl%d',varargin{1}),pt(1),sprintf('t%d',varargin{1}),pt(2));
		
			
		%% Move parabola line
		case 'l'
			
			%% Get measurement appdata
			meas=getappdata(hObject,'meas');
			
			%% Calculate delta t
			dt=pt(2)-([1,pt(1),pt(1)^2]*corrs.curv.parab.coeffs);

			%% Translate parabola
			t_plus_dt=([ones(3,1),corrs.curv.parab.wls',(corrs.curv.parab.wls').^2]*corrs.curv.parab.coeffs)+dt;
			if(~any(t_plus_dt>max(meas.t)) && ~any(t_plus_dt<min(meas.t)))
				set(corrs.curv.parab,'t1',corrs.curv.parab.t1+dt,'t2',corrs.curv.parab.t2+dt,'t3',corrs.curv.parab.t3+dt);
			end
			
	end

end

function trplGUI_figGUI_Load(hObject,eventdata,file_path)

	%% Get handles
	handles=guidata(hObject);
	
	%% Initialise measurement appdata
	
		%% Load data
		[meas.data,meas.wl,meas.t,meas.stat,meas.path]=trplLoad(file_path);

		%% Set *.img flag
		if(isfield(meas.stat,'Application'))														% Dirty check for *.img files
			meas.ITEX=true;
		else
			meas.ITEX=false;
		end

	
	%% Save appdata
	setappdata(handles.figGUI,'meas',meas);

	%% Plot data
	trplGUI_figGUI_Replot(hObject);
	
	%% Enable edit text boxes
	set([handles.etxtSelWl1,handles.etxtSelWl2,handles.etxtSelT1,handles.etxtSelT2,handles.etxtCBMin,handles.etxtCBMax],'Enable','On');
	
	%% Enable plot tools
	set([handles.toolZoomIn,handles.toolZoomOut,handles.toolPan,handles.toolDataCursor],'Enable','On');
	
	%% Add to MRU list
	
		%% Get MRU parameters
		menFile.emptylen=7;														% Would be better to do this dynamically (using file uimenu CreateFcn?)
		MRU.pos=menFile.emptylen-1;												% MRU position in file menu
		MRU.max=10;																% Max MRU entries
		MRU.maxpathlen=20;
		menFile.children=get(handles.menFile,'Children');
		MRU.len=length(menFile.children)-menFile.emptylen;
		
		%% Update older entry indices
		if(MRU.len~=0)
			MRU.inds=MRU.pos+1:MRU.pos+MRU.len;
			MRU.hs=menFile.children(MRU.inds);
			for ind_MRU=1:MRU.len
				newtag=sprintf('menFileMRU%d',MRU.len-ind_MRU+2);
				set(MRU.hs(ind_MRU),'Tag',newtag,'Label',strrep(get(MRU.hs(ind_MRU),'Label'),sprintf('%d: ',MRU.len-ind_MRU+1),sprintf('%d: ',MRU.len-ind_MRU+2)),'Separator','off');
				handles.(newtag)=MRU.hs(ind_MRU);
			end
		end
		
		%% Add newest entry
		if(length(meas.path)>MRU.maxpathlen)
			short_path=sprintf('...%s',meas.path(end-MRU.maxpathlen+1:end));
		else
			short_path=meas.path;
		end
		MRU.newest=uimenu('Parent',handles.menFile,'Tag','menFileMRU1','Label',['1: ',short_path],'Separator','on','UserData',meas.path,'Callback',{@trplGUI_figGUI_Load,meas.path});			% Automatically adds at children(1)
		menFile.children=[menFile.children(1:MRU.pos+MRU.len);MRU.newest;menFile.children(MRU.pos+MRU.len+1:end)];
		
		%% Remove excess entries
		if(length(menFile.children)>(menFile.emptylen+MRU.max))
			delete(menFile.children(MRU.pos+1));
			menFile.children(MRU.pos+1)=[];
		end
		
		%% Update file menu
		set(handles.menFile,'Children',menFile.children);
		
% 		handles.menFileMRU1=MRU.newest;
% 		guidata(handles.figGUI,handles);											% This doesn't seem to be working...

	%% Set window title and status bar string
	set(handles.figGUI,'Name',['trplGUI | ',meas.path]);
	[file_dir,file_name,file_ext]=fileparts(meas.path);
	if(meas.ITEX)
		stat_str=[file_name,file_ext,' | ',meas.stat.Application.Date,' ',meas.stat.Application.Time,' | ',meas.stat.Streakcamera.PluginName,' | '];
		switch(meas.stat.Acquisition.AcqMode)
			case '3'
				stat_str=[stat_str,'Photon Counting | '];
			case '4'
				stat_str=[stat_str,'Analogue Integration | '];
			otherwise
				stat_str=[stat_str,'Unknown Acquisition | '];
		end
		stat_str=[stat_str,meas.stat.Acquisition.NrExposure,' Exp @ ',meas.stat.Acquisition.ExposureTime,' | MCP Gain: ',meas.stat.Streakcamera.MCPGain,' | Time Range: ',meas.stat.Streakcamera.TimeRange];
	else
		stat_str=[file_name,file_ext];
	end
	set(handles.stxtStat,'String',stat_str);
	
	%% Clear any previous undo stack
	obj_udrd=getappdata(handles.figGUI,'uitools_FigureToolManager');
	if(~isempty(obj_udrd))
		obj_udrd.CommandManager.empty;
	end
	
	%% Unsmudge
	trplGUI_figGUI_Dirty(handles.figGUI,false);

end

function ver_current = trplGUI_figGUI_Current()

	ver_current=regexp(trplVer(),'v(\d)+\.(\d)+\.*(\d)*','tokens');
	ver_current=str2double(ver_current{1});
	ver_current(isnan(ver_current))=0;

end

function ver_latest = trplGUI_figGUI_Latest()

	%% Get google code downloads page source
	[source,status]=urlread('http://code.google.com/p/ma-code/downloads/list');
	if(~status)
		ver_latest=[0,0,0];
		return
	end
	
	%% Parse available trpl version numbers
	str_href_root='href="//ma-code.googlecode.com/files/';
	if(isdeployed())
		str_dist_root='trplGUI_Win%28x86%29_v';
	else
		str_dist_root='trpl_v';
	end
	str_regexp=[strrep(str_href_root,'.','\.'),str_dist_root,'(\d)+\.(\d)+\.*(\d)*\.zip"'];
	vers=cellfun(@(x) str2double(x),regexp(source,str_regexp,'tokens'),'UniformOutput',false);
	vers=cell2mat(vers');
	vers(isnan(vers))=0;
	vers=sortrows(vers,[-1,-2]);
	ver_latest=vers(1,:);
	
end

function handle = trplGUI_figGUI_CreateJButton(props)

	handle=uicontrol(props.Parent,'Style','pushbutton','String',props.String,'BackgroundColor',props.BackgroundColor,'Units',props.Units,'Position',props.Position,'Callback',{@(hObject,eventdata,href) web(href,'-browser'),props.HRef});
	jhandle=java(findjobj(handle,'nomenu'));
	jhandle.setBorder([]);
	jhandle.setCursor(java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));

end		

function trplGUI_figGUI_PlotCB(hObject)

	%% Get handles and CLim
	handles=guidata(hObject);
	clim=get(handles.axMain,'CLim');
	
	%% Update etxt boxes
	set(handles.etxtCBMin,'String',sprintf('%g',clim(1)));
	set(handles.etxtCBMax,'String',sprintf('%g',clim(2)));
	
	%% Update colorbar tick labels
	clim_quart=diff(clim)/4;
	clim_quarts=clim(1)+[clim_quart,2*clim_quart,3*clim_quart];
	set(handles.axCB,'YTickLabel',sprintf('%g|%g|%g',clim_quarts));
	
	%% Update handles
	guidata(hObject,handles);
	
end

function tf = trplGUI_figGUI_Dirty(hObject,varargin)

	%% Get handles and state appdatae
	handles=guidata(hObject);
	state=getappdata(handles.figGUI,'state');
	
	%% Get dirty state
	if(isempty(varargin))
		
		tf=state.dirty;
		
	%% Set dirty state
	else
		
		%% Valid value
		if(islogical(varargin{1}))
			
			items=[handles.menFileSave,handles.toolSave];
			if(varargin{1})
				set(items,'Enable','On');									% Enable dirty items
			else
				set(items,'Enable','Off');									% Disable dirty items
			end
			state.dirty=varargin{1};
			setappdata(handles.figGUI,'state',state);
			tf=true;
		
		%% Invalid value
		else		
			tf=false;
		end
		
	end
	
end


%%-- trplGUI_etxt* --%%

function trplGUI_etxt_CreateFcn(hObject, eventdata, handles)

	%% Match OS background colour
	if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
		set(hObject,'BackgroundColor','white');
	end

end

function trplGUI_etxt_FocusGainedCallback(jhObject, jeventdata)

	%% Select all text in edit text
	set(jhObject,'selectionStart',0);
	set(jhObject,'selectionEnd',length(get(jhObject,'text')));

end

function trplGUI_etxtSel_Callback(hObject, eventdata, handles)

	%% Determine edit box type from tag
	prop=regexp(get(hObject,'Tag'),'(?<=etxtSel)(Wl1|Wl2|T1|T2)','match');		% Change here if tag names change
	prop=lower(prop{1});
	if(isempty(prop))
		throw(MException('trplGUI:etxtSel:BadTag','Tag value was not recognised. Valid tags: etxtSel(Wl1|Wl2|T1|T2). Received: %s',get(hObject,'Tag')));
	end
	
	%% Get new value
	value=str2double(get(hObject,'String'));
	
	%% Get selections appdata
	sels=getappdata(gcbf,'sels');
	
	%% No previous selection exists
	if(isempty(sels))
		
		%% Bad value - reset to property name
		if(~isfinite(value))
			set(hObject,'String',prop);
		%% Good value - create selection
		else
			warning('off','trplGUI:Sel:NoFigGUIHandle');					% Switch off no figGUI warning
			sels=trplGUIClasses.Sel();										% Creates selection with measurement limits
			warning('on','trplGUI:Sel:NoFigGUIHandle');
			axis=regexp(prop,'(wl|t)','match');
			axis=axis{1};
			set(sels,[axis,'1'],value,[axis,'2'],value);
		end
		
	%% Previous selection exists
	else
		
		%% Bad value - reset to previous value
		if(~isfinite(value))
			format='%4.3f';													% Add to options
			set(hObject,'String',num2str(sels.(prop),format));
		%% Good value - modify selection
		else
			sels.(prop)=value;
		end
		
	end
	
	%% Update selections appdata
	setappdata(gcbf,'sels',sels);

end

function trplGUI_etxtCB_Callback(hObject, eventdata, handles, flag_max)

	%% Get appdata
	meas=getappdata(handles.figGUI,'meas');

	%% Store old value
	val_old=get(handles.axMain,'CLim');
	if(flag_max)
		val_opp=val_old(1);
		val_old=val_old(2);
	else
		val_opp=val_old(2);
		val_old=val_old(1);
	end

	%% Get new value
	val_new=str2double(get(hObject,'String'));
	
	%% Catch bad values
	if(isnan(val_new))
		val_new=val_old;								% Not a number - reset to old value
	else
		if(val_new==val_opp)
			if(flag_max)
				val_new=val_opp*1.01;					% Max same as min - increment
			else
				val_new=val_opp*0.99;					% Min same as max - subtract
			end
		end
	end
	
	%% Set CLim
	clim=sort([val_opp,val_new]);
	set(handles.axMain,'CLim',clim);
	
	%% Update colorbar
	trplGUI_figGUI_PlotCB(hObject);
	
	%% Update handles
	guidata(hObject,handles);

end


%%-- trplGUI_men* --%%

function trplGUI_menFile_Callback(hObject, eventdata, handles)

	%% Toggle file menu items on measurement exist
	if(isappdata(gcbf,'meas'))
		set([handles.menFileExpASCII;handles.menFileExpImg;handles.menFileSaveAs],'Enable','On');
				
		%% Enable export to workspace only if source distribution
		if(~isdeployed())
			set(handles.menFileExpWS,'Enable','On');
		end
		
	else
		set([handles.menFileExpASCII;handles.menFileExpWS;handles.menFileExpImg;handles.menFileSaveAs],'Enable','Off');
		set([get(handles.menFileExpASCII,'Children');get(handles.menFileExpWS,'Children')],'Enable','Off');
	end
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_menFileOpen_Callback(hObject, eventdata, handles)

	%% Determine previous path
	prev_path=[];
	if(isappdata(gcbf,'meas'))
		meas=getappdata(gcbf,'meas');
		prev_path=meas.path;
	end

	%% Load TRPL measurement data using GUI file selection
	exts={'*.img','ITEX image (*.img)';'*.dac','Calibrated ASCII image (*.dac)';'*.*','All Files (*.*)'};
	[file_name,file_dir]=uigetfile(exts,'Open TRPL data',prev_path);
	if(file_name~=0)
		trplGUI_figGUI_Load(handles.figGUI,[],[file_dir,file_name]);
	end
		
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_menFileSave_Callback(hObject, eventdata, handles, flag_saveas)

	%% Get measurement and uiundo app data
	meas=getappdata(gcbf,'meas');
	urdo=getappdata(handles.figGUI,'uitools_FigureToolManager');
	if(isempty(urdo))
		urdo=uitools.FigureToolManager(handles.figGUI);
		setappdata(handles.figGUI,'uitools_FigureToolManager',urdo);
	end
	
	%% Determine save as flag
	if(flag_saveas)
		file_path=fileparts(meas.path);
		if(isempty(file_path))
			file_path='.';												% Relative path
		end
	else
		file_path=meas.path;
	end
	
	%% Save data
	try
		trplSave(meas.data,meas.wl,meas.t,meas.stat,file_path);
	catch ME
		if(strcmp(ME.identifier,'trpl:SaveITEX:Stat'))
			errordlg('File cannot be saved in ITEX format as it does not contain a full status structure. This is probably because it was loaded in ASCII (*.dac, *.txt, *.asc) format. Consider saving the file in ASCII format or reloading it in ITEX format.','trplGUI - Cannot save to ITEX','modal');
			return;
		else
			ME.rethrow;
		end
	end
	
	%% Unsmudge
	trplGUI_figGUI_Dirty(handles.figGUI,false);
	
	%% Clear uiundo stack
	urdo.CommandManager.empty;
	
	
end

function trplGUI_menFileExit_Callback(hObject, eventdata, handles)

	%% Exit GUI
	close(handles.figGUI);

end

function trplGUI_menFileExp_Callback(hObject, eventdata, handles)

	%% Enable appropriate exports
	if(~isempty(getappdata(gcbf,'sels')))
		
		%% Enable all exports
		set(get(hObject,'Children'),'Enable','On');
	
	else
		
		%% Disable all exports
		set(get(hObject,'Children'),'Enable','Off');
		
		%% Enable only main data exports
		set(findobj(get(hObject,'Children'),'Tag',[get(hObject,'Tag'),'Data'],'-or','Tag',[get(hObject,'Tag'),'DataN']),'Enable','On');
	
	end
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_menFileExpImg_Callback(hObject, eventdata, handles)

	%% Get measurement appdata
	meas=getappdata(gcbf,'meas');
	
	%% Get file name
	exts={'*.ai','Adobe Illustrator `88 (*.ai)';'*.bmp','Windows bitmap (*.bmp)';'*.emf','Enhanced metafile (*.emf)';'*.eps','EPS Level 1 (*.eps)';'*.fig','MATLAB figure (*.fig)';'*.jpg','JPEG image (*.jpg)';'*.m','MATLAB file (*.m)';'*.pbm','Portable bitmap (*.pbm)';'*.pcx','Paintbrush 24-bit (*.pcx)';'*.pdf','Portable Document Format (*.pdf)';'*.pgm','Portable Graymap (*.pgm)';'*.png','Portable Network Graphics (*.png)';'*.ppm','Portable Pixmap (*.ppm)';'*.tif','TIFF image, compressed (*.tif)'};
	default.format='png';				% Optionalise
	default.extmask=strcmp(['*.',default.format],exts(:,1));
	exts=[exts(default.extmask,:);exts(~default.extmask,:)];
	[default.path,default.name]=fileparts(meas.path);
	[file.name,file.path,file.ext]=uiputfile(exts,'Export to image',sprintf('%s%s%s.%s',default.path,filesep,default.name,default.format));
	
	if(any(file.path))
		
		%% Create print figure
		h_fig=figure('Colormap',get(handles.figGUI,'Colormap'),'Visible','off');
		h_ax=copyobj(handles.axMain,h_fig);
		set(h_ax,'Units','normalized','OuterPosition',[0,0,1,1]);
		
		%% Export
		saveas(h_fig,[file.path,file.name],strrep(exts{file.ext,1},'*.',''));
		
		%% Close print figure
		close(h_fig);
		
	end

end

function trplGUI_menEdit_Callback(hObject, eventdata, handles)

	%% Toggle edit menu items on measurement exist
	if(isappdata(gcbf,'meas'))
		set(handles.menEditCopy,'Enable','On');
	else
		set(handles.menEditCopy,'Enable','Off');
		set(get(handles.menEditCopy,'Children'),'Enable','Off');
	end

	%% Toggle undo/redo menu items on if action exists
	obj_udrd=getappdata(gcbf,'uitools_FigureToolManager');
	if(~isempty(obj_udrd))
		%% Undo
		if(~isempty(obj_udrd.CommandManager.peekundo))
			set(handles.menEditUndo,'Enable','On');
			names=get(obj_udrd.CommandManager.UndoStack,'Name');
			if(iscell(names))
				%% Multiple actions
				set(handles.menEditUndo,'Label',['Undo ',names{end}]);
			else
				%% Single action
				set(handles.menEditUndo,'Label',['Undo ',names]);
			end
		else
			set(handles.menEditUndo,'Enable','Off');
			set(handles.menEditUndo,'Label','Undo');
		end
		%% Redo
		if(~isempty(obj_udrd.CommandManager.peekredo))
			set(handles.menEditRedo,'Enable','On');
			names=get(obj_udrd.CommandManager.RedoStack,'Name');
			if(iscell(names))
				%% Multiple actions
				set(handles.menEditRedo,'Label',['Redo ',names{end}]);
			else
				%% Single action
				set(handles.menEditRedo,'Label',['Redo ',names]);
			end
		else
			set(handles.menEditRedo,'Enable','Off');
			set(handles.menEditRedo,'Label','Redo');
		end
	end
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_menEditUndo_Callback(hObject, eventdata, handles)

	obj_udrd=getappdata(gcbf,'uitools_FigureToolManager');
	if(~isempty(obj_udrd) && ~isempty(obj_udrd.CommandManager.peekundo))
		
		%% Execute undo
		uiundo(gcbf,'execUndo');
		
		%% Enable undo/redo actions
		set(handles.menEditUndo,'Enable','On');
		set(handles.menEditRedo,'Enable','On');
		
	end
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_menEditRedo_Callback(hObject, eventdata, handles)

	obj_udrd=getappdata(gcbf,'uitools_FigureToolManager');
	if(~isempty(obj_udrd) && ~isempty(obj_udrd.CommandManager.peekredo))
		
		%% Execute redo
		uiundo(gcbf,'execRedo');
		
		%% Enable undo/redo actions
		set(handles.menEditUndo,'Enable','On');
		set(handles.menEditRedo,'Enable','On');
		
	end

	%% Update handles
	guidata(hObject,handles);
	
end

function trplGUI_menEditCopy_Callback(hObject, eventdata, handles)

	%% Enable appropriate copies
	if(~isempty(getappdata(gcbf,'sels')))
		
		%% Enable all copies
		set(get(hObject,'Children'),'Enable','On');
	
	else
		
		%% Disable all copies
		set(get(hObject,'Children'),'Enable','Off');
		
		%% Enable only main data copies
		set(findobj(get(hObject,'Children'),'Tag',[get(hObject,'Tag'),'Data'],'-or','Tag',[get(hObject,'Tag'),'DataN']),'Enable','On');
	
	end
	
	%% Update handles
	guidata(hObject,handles);
	
end

function trplGUI_menView_Callback(hObject, eventdata, handles)



end

function trplGUI_menViewLog_Callback(hObject, eventdata, handles)



end

function trplGUI_menViewLogProf_Callback(hObject, eventdata, handles, flag_spec)
	

	%% Initialise lookup tables
	lut_chk={'off';'on'};
	lut_scale={'linear';'log'};
	lut_ax={'XScale';'YScale'};
	lut_h_ax=[handles.axProfKin;handles.axProfSpec];
	lut_h_men=[handles.menViewLogKin,handles.menViewLogSpec;handles.cmenLogKin,handles.cmenLogSpec];

	%% Get current log scale status
	tf_chk=strcmpi(get(hObject,'Checked'),'on');
	
	%% Toggle axis log scaling
	set(lut_h_ax(flag_spec+1),lut_ax{flag_spec+1},lut_scale{(~tf_chk)+1});
	
	%% Toggle menu item check marks
	set(lut_h_men(:,flag_spec+1),'Checked',lut_chk{(~tf_chk)+1});
	
	
% 	
% 	%% Toggle log scaling
% 	if(tf_chk)
% 		
% 		%% Toggle axis log scaling
% 		if(flag_spec)
% 			set(handles.axProfSpec,'YScale','linear');
% 		else
% 			set(handles.axProfKin,'XScale','linear');
% 		end
% 		
% 		%% Toggle menu item check marks
% 		set(hObject,'Checked','off');
% 		
% 	else
% 		
% 		%% Toggle axis log scaling
% 		if(flag_spec)
% 			set(handles.axProfSpec,'YScale','log');
% 		else
% 			set(handles.axProfKin,'XScale','log');
% 		end
% 		
% 		%% Toggle menu item check marks
% 		set(hObject,'Checked','on');
% 		
% 	end
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_menViewCM_Callback(hObject, eventdata, handles)

	%% Enable set full range if measurement exists
	if(isappdata(gcbf,'meas'))
		set(handles.menViewCMFull,'Enable','On');
	else
		set(handles.menViewCMFull,'Enable','Off');
	end
	
	%% Update handles
	guidata(hObject,handles);
	

end

function trplGUI_menViewCMEditor_Callback(hObject, eventdata, handles)

	%% Binary distribution - Show error dialog
	if(isdeployed())
		
		handles_fit.figErrBin=msgbox({'The Mathworks does not allow the colormap editor GUI to be used in a standalone application.';[];'The complete trpl toolset - which includes trplGUI with full colormap editing capabilities - can be downloaded from the trpl homepage. Please note that this requires an existing Matlab installation.';[];[];[];[]},'trplGUI - Colormap Editor GUI Compilation Prohibited','error','modal');
		pbtErrBin.Parent=handles_fit.figErrBin;
		pbtErrBin.HRef='http://code.google.com/p/ma-code/downloads/list';
		pbtErrBin.String=sprintf('<html><a href="%s">trpl Homepage</a></html>',pbtErrBin.HRef);
		pbtErrBin.BackgroundColor=get(0,'defaultUicontrolBackgroundColor');
		pbtErrBin.Units='Normalized';
		pbtErrBin.Position=[0.41,0.25,0.18,0.12];
		handles_fit.figErrBin=trplGUI_figGUI_CreateJButton(pbtErrBin);
		
	%% Source distribution - Call colormap editor
	else

		%% Call colormap editor
		colormapeditor(handles.figGUI);

		%% Update handles
		guidata(hObject,handles);
	
	end

end

function trplGUI_menViewCMFull_Callback(hObject, eventdata, handles)

	%% Get appdata
	meas=getappdata(handles.figGUI,'meas');

	%% Set CLim to full range
	set(handles.axMain,'CLim',[min(min(meas.data)),max(max(meas.data))]);
	
	%% Update colorbar
	trplGUI_figGUI_PlotCB(hObject);
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_menCorr_Callback(hObject, eventdata, handles)

	%% Toggle correction menu items on measurement exist
	if(isappdata(gcbf,'meas'))
		
		%% Toggle menu items on open correction dialogs
		if(all(~isfield(handles,{'figCorrT0','figCorrCurv','figCorrSubRef'})))
			set([handles.menCorrT0;handles.menCorrCurv;handles.menCorrSub],'Enable','On');
		else
			set([handles.menCorrT0;handles.menCorrCurv;handles.menCorrSub],'Enable','Off');
		end
		
	else
		
		%% Disable all corrections
		set(get(hObject,'Children'),'Enable','Off');
		
	end
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_menCorrT0_Callback(hObject, eventdata, handles)
	
	%% Get appdata
	sels=getappdata(gcbf,'sels');
	meas=getappdata(gcbf,'meas');
	corrs=getappdata(gcbf,'corrs');
	
	%% Warn on previous t0 correction
	if(~isempty(meas.stat.trplGUI.CorrT0))
		ret=questdlg('t0 correction has already been performed on this data. Do you still wish to perform further t0 correction?','trplGUI - Previous t0 correction','Yes','No','No');
		if(isempty(ret) || strcmp(ret,'No'))
			return
		end
	end
			
	%% Disable trplGUI controls which can change data
	set([handles.menFileOpen,handles.toolOpen],'Enable','Off');
	
	%% Handle current selections
	if(~isempty(sels))
		sel_coords=[sels.wls;sels.ts]';
		sels.delete();
		setappdata(handles.figGUI,'sels',[]);
	else
		sel_coords=[];
	end
	
	%% Define number of spectral intervals
	spec_ints.number=5;																		% Allow this to be set (automatically/manually?)
	
	%% Create t0 correction dialog
	
		%% Define figure layout
		fig.margins=struct('top',7.5,'bottom',0,'left',10,'right',10);
		pans.margins=struct('horizontal',10,'top',25,'bottom',10);
		etxts=struct('dims',[51,26],'spacing',10,'jhandles',[]);
		pbuts=struct('margins',struct('horizontal',0,'vertical',7.5),'dims',[69,22.75],'spacing',10);

		%% Create figure
		fig.handle=figure('Color',get(handles.figGUI,'Color'),'DockControls','off','HandleVisibility','callback','IntegerHandle','off','NumberTitle','off','Name','trplGUI - t0 Correction','Units','pixels','Menubar','none','Toolbar','none','Visible','off');
		handles.figCorrT0=fig.handle;
		guidata(hObject,handles);								% Update handles
		fig.position=get(fig.handle,'Position');
		fig.position(3:4)=[fig.margins.left+fig.margins.right+(2*pans.margins.horizontal)+(spec_ints.number*etxts.dims(1))+((spec_ints.number-1)*etxts.spacing),fig.margins.top+fig.margins.bottom+(2*pans.margins.top)+(2*pans.margins.bottom)+(2*pbuts.margins.vertical)+(2*etxts.dims(2))+pbuts.dims(2)];
		set(fig.handle,'Position',fig.position,'CloseRequestFcn',{@trplGUICorrT0_Close,guidata(hObject)});
		
		%% Create push-buttons
		pbutCancel.position=[fig.position(3)-fig.margins.right-pbuts.margins.horizontal-pbuts.dims(1),fig.margins.bottom+pbuts.margins.vertical,pbuts.dims];
		pbutCancel.handle=uicontrol('Style','pushbutton','Parent',fig.handle,'String','Cancel','Units','pixels','Position',pbutCancel.position,'Callback',{@trplGUICorrT0_Close,guidata(hObject)});
		uicontrol(pbutCancel.handle);
		pbutOk.position=[pbutCancel.position(1)-pbuts.spacing-pbuts.dims(1),pbutCancel.position(2),pbuts.dims];
		pbutOk.handle=uicontrol('Style','pushbutton','Parent',fig.handle,'String','OK','Units','pixels','Position',pbutOk.position,'Callback',{@trplGUICorrT0_pbutOk_Callback,guidata(hObject)});

		%% Create panels
		panInt.position=[fig.margins.left,fig.margins.bottom+(2*pbuts.margins.vertical)+pbuts.dims(2)+pans.margins.top+pans.margins.bottom+etxts.dims(2),(2*pans.margins.horizontal)+(spec_ints.number*etxts.dims(1))+((spec_ints.number-1)*etxts.spacing),etxts.dims(2)+pans.margins.top+pans.margins.bottom];
		panInt.handle=uipanel('Parent',fig.handle,'Title','Interval t0','Units','pixels','Position',panInt.position);
		panAvg.position=[panInt.position(1),fig.margins.bottom+(2*pbuts.margins.vertical)+pbuts.dims(2),panInt.position(3:4)];
		panAvg.handle=uipanel('Parent',fig.handle,'Title','Average t0','Units','pixels','Position',panAvg.position);

		%% Create edit text boxes
		etxtInt(1:spec_ints.number)=struct('position',zeros(1,4),'handle',[]);
		for ind_etxt=1:spec_ints.number
			ind_int=spec_ints.number-ind_etxt+1;
			etxtInt(ind_int).position=[pans.margins.horizontal+((ind_etxt-1)*(etxts.dims(1)+etxts.spacing)),pans.margins.bottom,etxts.dims];
			etxtInt(ind_int).handle=uicontrol('Style','edit','Parent',panInt.handle,'Units','pixels','Position',etxtInt(ind_int).position,'Tag',sprintf('etxtInt%d',ind_int),'String',sprintf('t0_%d',ind_int),'Callback',{@trplGUICorrT0_etxtInt_Callback,guidata(hObject)});
		end
		etxtAvg.position=[pans.margins.horizontal,pans.margins.bottom,etxts.dims];
		etxtAvg.handle=uicontrol('Style','edit','Parent',panAvg.handle,'Units','pixels','Position',etxtAvg.position,'Tag','etxtAvg','String','t0','Callback',{@trplGUICorrT0_etxtAvg_Callback,guidata(hObject)});
		
		%% Set edit text box backgrounds
		if(ispc && isequal(get([[etxtInt.handle],etxtAvg.handle],'BackgroundColor'), repmat({get(0,'defaultUicontrolBackgroundColor')},spec_ints.number+1,1)))
			set([[etxtInt.handle],etxtAvg.handle],'BackgroundColor','white');
		end
		
		%% Set keypress function
		set([fig.handle,pbutOk.handle,pbutCancel.handle,etxtInt.handle,etxtAvg.handle],'KeyPressFcn',{@trplGUICorrT0_KeyPress_Callback,guidata(hObject)});
	
	%% Get kinetic profiles
	lims=struct('wl',[min(meas.wl),max(meas.wl)],'t',[min(meas.t),max(meas.t)]);
	spec_ints.width=diff(lims.wl)./spec_ints.number;
	spec_ints.wls=lims.wl(1)+([(0:spec_ints.number-1)',(1:spec_ints.number)'].*(spec_ints.width));
	spec_ints.kins.raw=trplGetKin(meas.data,meas.wl,spec_ints.wls)';
	spec_ints.kins.proc=(((-spec_ints.kins.raw)./repmat(max(spec_ints.kins.raw,[],2),1,size(spec_ints.kins.raw,2))).*0.9.*spec_ints.width)+repmat(spec_ints.wls(:,2),1,size(spec_ints.kins.raw,2));
	
	%% Define plot object options
	opts.avg.color=ones(1,3);
	opts.kins.color=hsv2rgb(rgb2hsv(opts.avg.color).*[1,1,0.5]);
	opts.lines.color=opts.kins.color;
	opts.lines.style=':';
	opts.pts.color=opts.kins.color;
	opts.pts.marker='s';
	opts.pts.pointer='top';
	opts.divs.color=opts.kins.color;
	
	%% Loop through intervals
	for ind_int=1:spec_ints.number;
	
		%% Plot intervals and kinetic profiles
		if(ind_int<spec_ints.number)
			spec_ints.h.divs(ind_int)=line('Parent',handles.axMain,'XData',[spec_ints.wls(ind_int,2),spec_ints.wls(ind_int,2)],'YData',[lims.t(1),lims.t(2)],'Color',opts.divs.color);
		end
		spec_ints.h.kins(ind_int)=line('Parent',handles.axMain,'XData',(spec_ints.kins.proc(ind_int,:))','YData',meas.t,'Color',opts.kins.color);
		
		%% Find interval t0 values
		try
			spec_ints.t0(ind_int)=trplT0Find(spec_ints.kins.raw(ind_int,:)',meas.t,'draw',false);
		catch ME
			warning(ME.identifier,'%s. Setting t0 to temporal mid-point',ME.message);
			spec_ints.t0(ind_int)=(max(meas.t)-min(meas.t))./2;											% Set t0 to middle of time axis on trplT0Find fail
		end
		
		%% Plot interval t0 lines
		spec_ints.h.lines(ind_int)=line('Parent',handles.axMain,'XData',spec_ints.wls(ind_int,:),'YData',[spec_ints.t0(ind_int),spec_ints.t0(ind_int)],'Color',opts.lines.color,'LineStyle',opts.lines.style);
		
	end	
	
	%% Average t0 points
	spec_ints.avg=mean(spec_ints.t0);
	
	%% Plot average t0 line
	spec_ints.h.avg=trplGUIClasses.Line([lims.wl(1),spec_ints.avg],[lims.wl(2),spec_ints.avg],handles.figGUI,'etxtt1',etxtAvg.handle,'LineColor',opts.avg.color,'Pointer','top');	% Order should be: lines, avg, pts,
	
	%% Plot interval t0 points
	for ind_int=1:spec_ints.number;
		spec_ints.h.pts(ind_int)=trplGUIClasses.Pt([mean(spec_ints.wls(ind_int,:)),spec_ints.t0(ind_int)],handles.figGUI,'etxtt',etxtInt(ind_int).handle,'Marker',opts.pts.marker,'MarkerEdgeColor',opts.pts.color,'MarkerFaceColor',opts.pts.color,'Pointer',opts.pts.pointer);
	end
		
	%% Disable point and line bound warnings
	warning('off','trplGUI:Pt:BadWl');
	warning('off','trplGUI:Pt:BadT');
	warning('off','trplGUI:Line:BadWl1');
	warning('off','trplGUI:Line:BadWl2');
	warning('off','trplGUI:Line:BadT1');
	warning('off','trplGUI:Line:BadT2');
	
	%% Store appdata
	corrs.t0=struct('pts',spec_ints.h.pts,'lines',spec_ints.h.lines,'avg',spec_ints.h.avg,'sel',sel_coords);
	setappdata(gcbf,'corrs',corrs);
	
	%% Set mouse button functions (reset to sel functions on completion of t0)
	set(gcbf,'WindowButtonDownFcn',@trplGUI_figGUI_WindowButtonDownT0_Callback);
	
	%% Make dialog visible
	set(fig.handle,'Visible','on');
	
	%% Set edit text box highlight on focus
	etxts.jhandles=findjobj(fig.handle,'class','edit');
	set(etxts.jhandles,'FocusGainedCallback',@trplGUI_etxt_FocusGainedCallback);

end

function trplGUI_menCorrT0_UdRd(handles, t0)

	%% Get measurement, selections and uiundo appdata
	meas=getappdata(handles.figGUI,'meas');
	sels=getappdata(handles.figGUI,'sels');
	urdo=getappdata(handles.figGUI,'uitools_FigureToolManager');
	if(isempty(urdo))
		urdo=uitools.FigureToolManager(handles.figGUI);
		setappdata(handles.figGUI,'uitools_FigureToolManager',urdo);
	end
	
	%% Translate temporal axis
	meas.t=meas.t-t0;
	if(t0<0)
		meas.stat.trplGUI.CorrT0=[];										% Undo
		if(isempty(get(urdo.CommandManager.peekundo)))
			trplGUI_figGUI_Dirty(handles.figGUI,false);
		end
	else
		meas.stat.trplGUI.CorrT0=sprintf('%4.3f',t0);						% Redo
		trplGUI_figGUI_Dirty(handles.figGUI,true);
	end
	setappdata(handles.figGUI,'meas',meas);
	
	
	
	
	%% Translate selection data
	if(~isempty(sels))
		set(sels,'t1',sels.t1-t0,'t2',sels.t2-t0);
		setappdata(handles.figGUI,'sels',sels);
	end
	
	%% Replot data
	trplGUI_figGUI_Replot(handles.figGUI);

end

function trplGUI_menCorrCurv_Callback(hObject, eventdata, handles)

	
	%% Get appdata
	sels=getappdata(gcbf,'sels');
	meas=getappdata(gcbf,'meas');
	corrs=getappdata(gcbf,'corrs');
	
	%% Warn on previous curvature correction
	if(~isempty(meas.stat.trplGUI.CorrCurv))
		ret=questdlg('Curvature correction has already been performed on this data. Do you still wish to perform further curvature correction?','trplGUI - Previous curvature correction','Yes','No','No');
		if(isempty(ret) || strcmp(ret,'No'))
			return
		end
	
	%% Warn on not synchroscan
	elseif(meas.ITEX && ~strcmp(meas.stat.Streakcamera.PluginName,'M5675'))
		ret=questdlg('This data was not taken with the synchroscan plugin (M5675). Curvature correction should not be necessary. Do you still wish to perform curvature correction?','trplGUI - Non-synchroscan curvature correction','Yes','No','No');
		if(isempty(ret) || strcmp(ret,'No'))
			return
		end
	end
	
	%% Disable trplGUI controls which can change data
	set([handles.menFileOpen,handles.toolOpen],'Enable','Off');
	
	%% Handle current selections
	if(~isempty(sels))
		sel_coords=[sels.wls;sels.ts]';
		sels.delete();
		setappdata(handles.figGUI,'sels',[]);
	else
		sel_coords=[];
	end
	
	%% Define number of spectral intervals
	spec_ints.number=5;																		% Allow this to be set (automatically/manually?)
	
	%% Create curvature correction dialog
	
		%% Define figure layout
		fig.margins=struct('top',7.5,'bottom',0,'left',10,'right',10);
		pans.margins=struct('horizontal',10,'top',25,'bottom',10);
		etxts=struct('dims',[51,26],'spacing',struct('horizontal',10,'vertical',5),'jhandles',[]);
		pbuts=struct('margins',struct('horizontal',0,'vertical',7.5),'dims',[69,22.75],'spacing',10);

		%% Create figure
		fig.handle=figure('Color',get(handles.figGUI,'Color'),'DockControls','off','HandleVisibility','callback','IntegerHandle','off','NumberTitle','off','Name','trplGUI - Curvature Correction','Units','pixels','Menubar','none','Toolbar','none','Visible','off');
		handles.figCorrCurv=fig.handle;
		guidata(hObject,handles);								% Update handles
		fig.position=get(fig.handle,'Position');
		fig.position(3:4)=[fig.margins.left+fig.margins.right+(2*pans.margins.horizontal)+(spec_ints.number*etxts.dims(1))+((spec_ints.number-1)*etxts.spacing.horizontal),fig.margins.top+fig.margins.bottom+(2*pans.margins.top)+(2*pans.margins.bottom)+(2*pbuts.margins.vertical)+etxts.spacing.vertical+(3*etxts.dims(2))+pbuts.dims(2)];
		set(fig.handle,'Position',fig.position,'CloseRequestFcn',{@trplGUICorrCurv_Close,guidata(hObject)});
		
		%% Create push-buttons
		pbutCancel.position=[fig.position(3)-fig.margins.right-pbuts.margins.horizontal-pbuts.dims(1),fig.margins.bottom+pbuts.margins.vertical,pbuts.dims];
		pbutCancel.handle=uicontrol('Style','pushbutton','Parent',fig.handle,'String','Cancel','Units','pixels','Position',pbutCancel.position,'Callback',{@trplGUICorrCurv_Close,guidata(hObject)});
		uicontrol(pbutCancel.handle);
		pbutOk.position=[pbutCancel.position(1)-pbuts.spacing-pbuts.dims(1),pbutCancel.position(2),pbuts.dims];
		pbutOk.handle=uicontrol('Style','pushbutton','Parent',fig.handle,'String','OK','Units','pixels','Position',pbutOk.position,'Callback',{@trplGUICorrCurv_pbutOk_Callback,guidata(hObject)});

		%% Create panels
		panInt.position=[fig.margins.left,fig.margins.bottom+(2*pbuts.margins.vertical)+pbuts.dims(2)+pans.margins.top+pans.margins.bottom+etxts.spacing.vertical+(2*etxts.dims(2)),(2*pans.margins.horizontal)+(spec_ints.number*etxts.dims(1))+((spec_ints.number-1)*etxts.spacing.horizontal),etxts.dims(2)+pans.margins.top+pans.margins.bottom];
		panInt.handle=uipanel('Parent',fig.handle,'Title','Interval t0','Units','pixels','Position',panInt.position);
		panParab.position=[panInt.position(1),fig.margins.bottom+(2*pbuts.margins.vertical)+pbuts.dims(2),panInt.position(3),panInt.position(4)+etxts.dims(2)+etxts.spacing.vertical];
		panParab.handle=uipanel('Parent',fig.handle,'Title','Parabola points','Units','pixels','Position',panParab.position);

		%% Create edit text boxes
		etxtInt(1:spec_ints.number)=struct('position',zeros(1,4),'handle',[]);
		for ind_etxt=1:spec_ints.number
			ind_int=spec_ints.number-ind_etxt+1;
			etxtInt(ind_int).position=[pans.margins.horizontal+((ind_etxt-1)*(etxts.dims(1)+etxts.spacing.horizontal)),pans.margins.bottom,etxts.dims];
			etxtInt(ind_int).handle=uicontrol('Style','edit','Parent',panInt.handle,'Units','pixels','Position',etxtInt(ind_int).position,'Tag',sprintf('etxtInt%d',ind_int),'String',sprintf('t0_%d',ind_int),'Callback',{@trplGUICorrCurv_etxtInt_Callback,guidata(hObject)});
		end
		etxtParab(2,3)=struct('position',zeros(1,4),'handle',[]);
		for ind_etxt=1:3
			etxtParab(2,ind_etxt).position=[pans.margins.horizontal+((ind_etxt-1)*(etxts.dims(1)+etxts.spacing.horizontal)),pans.margins.bottom,etxts.dims];
			etxtParab(2,ind_etxt).handle=uicontrol('Style','edit','Parent',panParab.handle,'Units','pixels','Position',etxtParab(2,ind_etxt).position,'Tag',sprintf('etxtParabT%d',4-ind_etxt),'String',sprintf('t%d',4-ind_etxt),'Callback',{@trplGUICorrCurv_etxtParab_Callback,guidata(hObject)});
		end
		for ind_etxt=1:3
			etxtParab(1,ind_etxt).position=[pans.margins.horizontal+((ind_etxt-1)*(etxts.dims(1)+etxts.spacing.horizontal)),pans.margins.bottom+etxts.dims(2)+etxts.spacing.vertical,etxts.dims];
			etxtParab(1,ind_etxt).handle=uicontrol('Style','edit','Parent',panParab.handle,'Units','pixels','Position',etxtParab(1,ind_etxt).position,'Tag',sprintf('etxtParabWl%d',4-ind_etxt),'String',sprintf('wl%d',4-ind_etxt),'Callback',{@trplGUICorrCurv_etxtParab_Callback,guidata(hObject)});
		end
		
		%% Set edit text box backgrounds
		if(ispc && isequal(get([[etxtInt.handle],reshape([etxtParab.handle],1,[])],'BackgroundColor'), repmat({get(0,'defaultUicontrolBackgroundColor')},spec_ints.number+6,1)))
			set([[etxtInt.handle],reshape([etxtParab.handle],1,[])],'BackgroundColor','white');
		end
		
		%% Set keypress function
		set([fig.handle,pbutOk.handle,pbutCancel.handle,etxtInt.handle,etxtParab.handle],'KeyPressFcn',{@trplGUICorrCurv_KeyPress_Callback,guidata(hObject)});
	
	%% Get kinetic profiles
	lims=struct('wl',[min(meas.wl),max(meas.wl)],'t',[min(meas.t),max(meas.t)]);
	spec_ints.width=diff(lims.wl)./spec_ints.number;
	spec_ints.wls=lims.wl(1)+([(0:spec_ints.number-1)',(1:spec_ints.number)'].*(spec_ints.width));
	spec_ints.kins.raw=trplGetKin(meas.data,meas.wl,spec_ints.wls)';
	spec_ints.kins.proc=(((-spec_ints.kins.raw)./repmat(max(spec_ints.kins.raw,[],2),1,size(spec_ints.kins.raw,2))).*0.9.*spec_ints.width)+repmat(spec_ints.wls(:,2),1,size(spec_ints.kins.raw,2));
	
	%% Determine spectrally weighted mean wavelengths
	spec_ints.specs.masks=sparse((repmat(meas.wl',spec_ints.number,1)<repmat(spec_ints.wls(:,2),1,length(meas.wl)))&(repmat(meas.wl',spec_ints.number,1)>repmat(spec_ints.wls(:,1),1,length(meas.wl))));
	spec_ints.specs.integ=repmat(trplGetSpec(meas.data,meas.t,lims.t),spec_ints.number,1);
	spec_ints.specs.wl=repmat((meas.wl'),spec_ints.number,1);
	spec_ints.specs.wl(~spec_ints.specs.masks)=0;
	spec_ints.specs.wl=sparse(spec_ints.specs.wl);
	spec_ints.specs.integ(~spec_ints.specs.masks)=0;
	spec_ints.specs.integ=sparse(spec_ints.specs.integ);
	spec_ints.wwl=sum(spec_ints.specs.wl.*spec_ints.specs.integ,2)./sum(spec_ints.specs.integ,2);
	
	%% Define plot object options
	opts.parab.line.color=ones(1,3);
	opts.parab.line.pointer='top';
	opts.parab.pt.color=opts.parab.line.color;
	opts.kin.color=hsv2rgb(rgb2hsv(opts.parab.line.color).*[1,1,0.5]);
	opts.div.color=opts.kin.color;
	opts.t0.line.color=opts.kin.color;
	opts.t0.line.style=':';
	opts.t0.pt.color=opts.kin.color;
	opts.t0.pt.marker='s';
	opts.t0.pt.pointer='top';
	
	%% Loop through intervals
	for ind_int=1:spec_ints.number;
	
		%% Plot intervals and kinetic profiles
		if(ind_int<spec_ints.number)
			spec_ints.h.divs(ind_int)=line('Parent',handles.axMain,'XData',[spec_ints.wls(ind_int,2),spec_ints.wls(ind_int,2)],'YData',[lims.t(1),lims.t(2)],'Color',opts.div.color);
		end
		spec_ints.h.kins(ind_int)=line('Parent',handles.axMain,'XData',(spec_ints.kins.proc(ind_int,:))','YData',meas.t,'Color',opts.kin.color);
		
		%% Find interval t0 values
		try
			spec_ints.t0(ind_int)=trplT0Find(spec_ints.kins.raw(ind_int,:)',meas.t,'draw',false);
		catch ME
			warning(ME.identifier,'%s. Setting t0 to temporal mid-point',ME.message);
			spec_ints.t0(ind_int)=(max(meas.t)-min(meas.t))./2;											% Set t0 to middle of time axis on trplT0Find fail
		end
		
		%% Plot interval t0 lines
		spec_ints.h.t0lines(ind_int)=line('Parent',handles.axMain,'XData',spec_ints.wls(ind_int,:),'YData',[spec_ints.t0(ind_int),spec_ints.t0(ind_int)],'Color',opts.t0.line.color,'LineStyle',opts.t0.line.style);
		
	end
	
	%% Fit parabola to t0 points
	parab.coeffs=[ones(spec_ints.number,1),spec_ints.wwl,(spec_ints.wwl).^2]\(spec_ints.t0');							% mldivide lsq solution
	
	%% Plot parabola
	parab.wl=lims.wl(1)+(((1:3)').*(diff(lims.wl)./4));
	parab.t=[ones(3,1),parab.wl,(parab.wl).^2]*parab.coeffs;
	parab.handle=trplGUIClasses.Parab([parab.wl(1),parab.t(1)],[parab.wl(2),parab.t(2)],[parab.wl(3),parab.t(3)],handles.figGUI,'etxtWl1',etxtParab(1,3).handle,'etxtWl2',etxtParab(1,2).handle,'etxtWl3',etxtParab(1,1).handle,'etxtT1',etxtParab(2,3).handle,'etxtT2',etxtParab(2,2).handle,'etxtT3',etxtParab(2,1).handle,'LineColor',opts.parab.line.color,'LinePointer',opts.parab.line.pointer,'PtFaceColor',opts.parab.pt.color,'PtEdgeColor',opts.parab.pt.color);
	
	%% Plot interval t0 points
	for ind_int=1:spec_ints.number;
		spec_ints.h.t0pts(ind_int)=trplGUIClasses.Pt([spec_ints.wwl(ind_int),spec_ints.t0(ind_int)],handles.figGUI,'etxtt',etxtInt(ind_int).handle,'Marker',opts.t0.pt.marker,'MarkerEdgeColor',opts.t0.pt.color,'MarkerFaceColor',opts.t0.pt.color,'Pointer',opts.t0.pt.pointer);
	end
		
	%% Disable point and line bound warnings
	warning('off','trplGUI:Pt:BadWl');
	warning('off','trplGUI:Pt:BadT');
	warning('off','trplGUI:Line:BadWl1');
	warning('off','trplGUI:Line:BadWl2');
	warning('off','trplGUI:Line:BadT1');
	warning('off','trplGUI:Line:BadT2');
	
	%% Store appdata	
	corrs.curv=struct('sel',sel_coords,'t0',struct('pts',spec_ints.h.t0pts,'lines',spec_ints.h.t0lines),'parab',parab.handle);
	setappdata(gcbf,'corrs',corrs);
	
	%% Set mouse button functions (reset to sel functions on completion of curv)
	set(gcbf,'WindowButtonDownFcn',@trplGUI_figGUI_WindowButtonDownCurv_Callback);

	%% Make dialog visible
	set(fig.handle,'Visible','on');
	
	%% Set edit text box highlight on focus
	etxts.jhandles=findjobj(fig.handle,'class','edit');
	set(etxts.jhandles,'FocusGainedCallback',@trplGUI_etxt_FocusGainedCallback);
	
end

function trplGUI_menCorrCurv_UdRd(handles, data, varargin)

	%% Get measurement and uiundo appdata
	meas=getappdata(handles.figGUI,'meas');
	urdo=getappdata(handles.figGUI,'uitools_FigureToolManager');
	if(isempty(urdo))
		urdo=uitools.FigureToolManager(handles.figGUI);
		setappdata(handles.figGUI,'uitools_FigureToolManager',urdo);
	end
	
	%% Perform undo/redo
	meas.data=data;
	if(isempty(meas.stat.trplGUI.CorrCurv))	
		meas.stat.trplGUI.CorrCurv=varargin{1};				% Redo
		trplGUI_figGUI_Dirty(handles.figGUI,true);
	else
		meas.stat.trplGUI.CorrCurv=[];						% Undo
		if(isempty(get(urdo.CommandManager.peekundo)))
			trplGUI_figGUI_Dirty(handles.figGUI,false);
		end
	end
	setappdata(handles.figGUI,'meas',meas);
	
	%% Replot data
	trplGUI_figGUI_Replot(handles.figGUI);
		
end

function trplGUI_menCorrSub_Callback(hObject, eventdata, handles)
end

function trplGUI_menCorrSubRef_Callback(hObject, eventdata, handles)

	%% Get appdata
	sels=getappdata(gcbf,'sels');
	meas=getappdata(gcbf,'meas');
	corrs=getappdata(gcbf,'corrs');
	
	%% Warn on previous reference image subtraction
	if(~isempty(meas.stat.trplGUI.CorrSubRef))
		ret=questdlg('Reference image subtraction has already been performed on this data. Do you still wish to perform further reference image subtraction?','trplGUI - Previous reference image subtraction','Yes','No','No');
		if(isempty(ret) || strcmp(ret,'No'))
			return
		end
	end
	
	%% Disable trplGUI controls which can change data
	set([handles.menFileOpen,handles.toolOpen],'Enable','Off');
	
	%% Create reference image subtraction dialog
		
		%% Define figure layout
		fig.margins=struct('top',7.5,'bottom',0,'left',10,'right',10);
		pans.margins=struct('horizontal',10,'top',25,'bottom',10);
		filesel=struct('etxt',struct('dims',[400,22.75],'jhandles',[]),'pbut',struct('dims',[69,22.75]),'spacing',10);
		chkbox=struct('dims',[100,35]);
		pbuts=struct('margins',struct('horizontal',0,'vertical',7.5),'dims',[69,22.75],'spacing',10);
		
		%% Create figure
		fig.handle=figure('Color',get(handles.figGUI,'Color'),'DockControls','off','HandleVisibility','callback','IntegerHandle','off','NumberTitle','off','Name','trplGUI - Reference Image Subtraction','Units','pixels','Menubar','none','Toolbar','none','Visible','off');
		handles.figCorrSubRef=fig.handle;
		guidata(hObject,handles);								% Update handles
		fig.position=get(fig.handle,'Position');
		fig.position(3:4)=[fig.margins.left+fig.margins.right+(2*pans.margins.horizontal)+filesel.etxt.dims(1)+filesel.spacing+filesel.pbut.dims(1),fig.margins.top+fig.margins.bottom+pans.margins.top+pans.margins.bottom+(2*pbuts.margins.vertical)+pbuts.dims(2)+chkbox.dims(2)+filesel.etxt.dims(2)];
		set(fig.handle,'Position',fig.position,'CloseRequestFcn',{@trplGUICorrSubRef_Close,guidata(hObject)});
		
		%% Create push-buttons
		pbutOk.position=[fig.position(3)-fig.margins.right-pbuts.margins.horizontal-pbuts.dims(1),fig.margins.bottom+pbuts.margins.vertical,pbuts.dims];
		pbutOk.handle=uicontrol('Style','pushbutton','Parent',fig.handle,'Tag','pbutOk','String','OK','Units','pixels','Position',pbutOk.position,'Enable','off','Callback',{@trplGUICorrSubRef_pbutOk_Callback,guidata(hObject)});
		pbutCancel.position=[pbutOk.position(1)-pbuts.spacing-pbuts.dims(1),pbutOk.position(2),pbuts.dims];
		pbutCancel.handle=uicontrol('Style','pushbutton','Parent',fig.handle,'Tag','pbutCancel','String','Cancel','Units','pixels','Position',pbutCancel.position,'Callback',{@trplGUICorrSubRef_Close,guidata(hObject)});

		%% Create panels
		panRef.position=[fig.margins.left,fig.margins.bottom+(2*pbuts.margins.vertical)+pbuts.dims(2),(2*pans.margins.horizontal)+filesel.etxt.dims(1)+filesel.spacing+filesel.pbut.dims(1),pans.margins.top+pans.margins.bottom+filesel.etxt.dims(2)+chkbox.dims(2)];
		panRef.handle=uipanel('Parent',fig.handle,'Tag','panRef','Title','Reference Image File','Units','pixels','Position',panRef.position);
		
		%% Create check-box
		chkboxClip.position=[pans.margins.horizontal,pans.margins.bottom,chkbox.dims];
		chkboxClip.handle=uicontrol('Style','checkbox','Parent',panRef.handle,'Tag','chkboxClip','String','Clip to zero','Units','pixels','Position',chkboxClip.position,'Min',0,'Max',1,'Value',0);

		%% Create file selector
		etxtFilesel.position=[pans.margins.horizontal,pans.margins.bottom+chkbox.dims(2),filesel.etxt.dims];
		etxtFilesel.handle=uicontrol('Style','edit','Parent',panRef.handle,'Tag','etxtFilesel','Units','pixels','Position',etxtFilesel.position,'String','','HorizontalAlignment','left','Callback',{@trplGUICorrSubRef_etxtFilesel_Callback,guidata(hObject)});
		if(ispc && isequal(get(etxtFilesel.handle,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor')))
			set(etxtFilesel.handle,'BackgroundColor','white');
		end
		uicontrol(etxtFilesel.handle);
		pbutFilesel.position=[pans.margins.horizontal+filesel.etxt.dims(1)+filesel.spacing,etxtFilesel.position(2),filesel.pbut.dims];
		pbutFilesel.handle=uicontrol('Style','pushbutton','Parent',panRef.handle,'Tag','pbutFilesel','String','Browse...','Units','pixels','Position',pbutFilesel.position,'Callback',{@trplGUICorrSubRef_pbutFilesel_Callback,guidata(hObject)});
		
		%% Set keypress function
		set([fig.handle,pbutOk.handle,pbutCancel.handle,chkboxClip.handle,etxtFilesel.handle,pbutFilesel.handle],'KeyPressFcn',{@trplGUICorrSubRef_KeyPress_Callback,guidata(hObject)});

	%% Make dialog visible
	set(fig.handle,'Visible','on');

end

function trplGUI_menCorrSubRef_UdRd(handles, path, clip)

	%% Get measurement and uiundo appdata
	meas=getappdata(handles.figGUI,'meas');
	urdo=getappdata(handles.figGUI,'uitools_FigureToolManager');
	if(isempty(urdo))
		urdo=uitools.FigureToolManager(handles.figGUI);
		setappdata(handles.figGUI,'uitools_FigureToolManager',urdo);
	end

	%% Try loading reference file
	try
		ref_data=trplLoad(path);
	catch ME
		errordlg(sprintf('Error opening reference image file (%s).\n%s.',path,ME.message),'trplGUI - Error opening reference image file','modal');
		return
	end
		
	%% Perform undo/redo
	if(isempty(meas.stat.trplGUI.CorrSubRef))
		meas.stat.trplGUI.CorrSubRef=[path,'|',sprintf('%d',clip.flag)];		% Redo
		meas.data=meas.data-ref_data;
		if(clip.flag)
			meas.data(meas.data<0)=0;
		end
		trplGUI_figGUI_Dirty(handles.figGUI,true);
	else
		meas.stat.trplGUI.CorrSubRef=[];										% Undo
		if(clip.flag)
			meas.data(clip.ind)=clip.val;
		end
		meas.data=meas.data+ref_data;
		if(isempty(get(urdo.CommandManager.peekundo)))
			trplGUI_figGUI_Dirty(handles.figGUI,false);
		end
	end
	setappdata(handles.figGUI,'meas',meas);
	
	%% Replot data
	trplGUI_figGUI_Replot(handles.figGUI);
	
end

function trplGUI_menFit_Callback(hObject, eventdata, handles)

	%% Toggle fitting menu items of profile exist
	if(~isempty(getappdata(gcbf,'sels')))
		
		%% Enable all exports
		set(get(handles.menFit,'Children'),'Enable','On');
	
	else
		
		%% Disable all exports
		set(get(handles.menFit,'Children'),'Enable','Off');
		
	end
	
	%% Update handles
	guidata(hObject,handles);
	
end

function trplGUI_menFitProf_Callback(hObject, eventdata, handles, flag_norm ,flag_spec)

	%% Binary distribution - Show error dialog
	if(isdeployed())
		
		handles_fit.figErrBin=msgbox({'The Mathworks does not allow the curve fitting toolbox GUI to be used in a standalone application. Please use the profile export function of trplGUI to export the desired profile to an external fitting program.';[];'Alternatively, the complete trpl toolset - which includes trplGUI with full fitting capabilities - can be downloaded from the trpl homepage. Please note that this requires an existing Matlab installation.';[];[];[];[]},'trplGUI - Curve Fitting Toolbox GUI Compilation Prohibited','error','modal');
		pbtErrBin.Parent=handles_fit.figErrBin;
		pbtErrBin.HRef='http://code.google.com/p/ma-code/downloads/list';
		pbtErrBin.String=sprintf('<html><a href="%s">trpl Homepage</a></html>',pbtErrBin.HRef);
		pbtErrBin.BackgroundColor=get(0,'defaultUicontrolBackgroundColor');
		pbtErrBin.Units='Normalized';
		pbtErrBin.Position=[0.41,0.25,0.18,0.12];
		handles_fit.figErrBin=trplGUI_figGUI_CreateJButton(pbtErrBin);
	
		
	%% Source distribution - Call cftool with profile
	else
		
		%% Get appdata
		meas=getappdata(gcbf,'meas');
		sels=getappdata(gcbf,'sels');
		
		%% Generate variable names for cftool
		[file_name,file_name]=fileparts(meas.path);
		var=genvarname(file_name,who);						%% Don't have to worry about naming conflicts as have functional scope
		
		%% Fit spectrum
		if(flag_spec)
			
			%% Get spectrum
			data_x=meas.wl;
			data_y=trplGetSpec(meas.data,meas.t,sels.ts);
			if(flag_norm)
				data_y=data_y./(repmat(max(abs(data_y),[],2),1,size(data_y,2)));
			end
			
			%% Generate variable name
			var_x=[var,'_wl'];
			var_y=[var,'_pl'];
			
		%% Fit kinetic
		else
			
			%% Get kinetic
			data_x=meas.t;
			data_y=trplGetKin(meas.data,meas.wl,sels.wls);
			if(flag_norm)
				data_y=data_y./(repmat(max(abs(data_y),[],1),size(data_y,1),1));
			end
			
			%% Generate variable name
			var_x=[var,'_t'];
			var_y=[var,'_pl'];
			
		end
		
		%% Call cftool
		eval(sprintf('%s=%s;',var_x,mat2str(data_x)));
		eval(sprintf('%s=%s;',var_y,mat2str(data_y)));
		eval(sprintf('cftool(%s,%s)',var_x,var_y));
		
	end
	
	%% Update handles
	guidata(hObject,handles);
	
end

function trplGUI_menHelp_Callback(hObject, eventdata, handles)
end

function trplGUI_menHelpAbout_Callback(hObject, eventdata, handles)

	%% Define figure parameters
	footer_height=0.25;
	footer_width=1;
	logo_height=1-footer_height;
	logo_width=0.3;
	text_height=1-footer_height;
	text_width=0.95-logo_width;
	color_aboutbg=[1,1,1];
	color_uibg=get(0,'defaultUicontrolBackgroundColor');
	
	%% Create dialog box
	handles_about.figGUIAbout=dialog('Units','pixels','Name','trplGUI - About','Color',color_aboutbg);
	figGUIAbout=get(handles_about.figGUIAbout);
	figGUIAbout.Position(3:4)=[600,275];
	figGUIAbout.WindowButtonDownFcn={@(hObject,eventdata) close(hObject)};
	set(handles_about.figGUIAbout,'Position',figGUIAbout.Position,'WindowButtonDownFcn',figGUIAbout.WindowButtonDownFcn);
	set(handles_about.figGUIAbout,'Units','Normalized');
	
	%% Create logo
	handles_about.axLogo=axes('Parent',handles_about.figGUIAbout,'Units','normalized','OuterPosition',[0,footer_height,logo_width,logo_height],'Position',[0,footer_height,logo_width,logo_height],'YDir','reverse','XLim',[-50,250],'YLim',[-20,620],'Visible','off');
	[logo,logo_cmap,logo_alpha]=imread('logo.png');
 	handle_about.imLogo=image('Parent',handles_about.axLogo,'CData',logo,'AlphaData',logo_alpha);
	
	%% Create main text
	handles_about.atxtboxAbout=annotation(handles_about.figGUIAbout,'textbox',[logo_width,footer_height,text_width,text_height],'HorizontalAlignment','left','VerticalAlignment','middle','EdgeColor','none','FontSize',8);
	atxtboxAbout=get(handles_about.atxtboxAbout);
	ver_current=trplGUI_figGUI_Current();
	ver_latest=trplGUI_figGUI_Latest();
	atxtboxAbout.String={sprintf('{\\color[rgb]{%f,%f,%f}\\fontsize{14}\\bfv%d.%d.%d}',color_uibg,ver_current(1),ver_current(2),ver_current(3));'{\fontsize{4} }'};
	if(ver_latest(1)>ver_current(1) || (ver_latest(1)==ver_current(1) && ver_latest(2)>ver_current(2)) || (ver_latest(1)==ver_current(1) && ver_latest(2)==ver_current(2) && ver_latest(3)>ver_current(3)))
		
		%% New version available
		atxtboxAbout.String=[atxtboxAbout.String;{sprintf('{\\it\\color[rgb]{%f,%f,%f}trplGUI is up to date}',color_aboutbg)}];				% Placeholder text
		pbtUpdates.Parent=handles_about.figGUIAbout;
		pbtUpdates.HRef='http://code.google.com/p/ma-code/downloads/list';
		pbtUpdates.String=sprintf('<html><a href="%s"><b>v%d.%d.%d available</b></a></html>',pbtUpdates.HRef,ver_latest(1),ver_latest(2),ver_latest(3));
		pbtUpdates.BackgroundColor=color_aboutbg;
		pbtUpdates.Units='Normalized';
		pbtUpdates.Position=[0.3117,0.7818,0.1500,0.0600];
		handles_about.pbtUpdates=trplGUI_figGUI_CreateJButton(pbtUpdates);
		
	else
		
		%% Up to date
		atxtboxAbout.String=[atxtboxAbout.String;{sprintf('{\\it\\color[rgb]{%f,%f,%f}trplGUI is up to date}',color_uibg)}];
		
	end
	atxtboxAbout.String=[atxtboxAbout.String;{[];[];
		'A graphical user interface for analysis of time-resolved photoluminescence data taken in the Chameleon Lab.';
		'Based on the trpl toolset.';[];[];
		'Authors:';'M.J.P. Alcocer:                                      ||'}];
	set(handles_about.atxtboxAbout,'String',atxtboxAbout.String);
	
	%% Create IIT e-mail button
	pbtMailIIT.Parent=handles_about.figGUIAbout;
	pbtMailIIT.HRef='mailto:marcelo.alcocer@iit.it';
	pbtMailIIT.String=sprintf('<html><a href="%s">marcelo.alcocer@iit.it</a></html>',pbtMailIIT.HRef);
	pbtMailIIT.BackgroundColor=color_aboutbg;
	pbtMailIIT.Units='Normalized';
	pbtMailIIT.Position=[0.4333,0.3199,0.1820,0.0664];
	handles_about.pbtMailIIT=trplGUI_figGUI_CreateJButton(pbtMailIIT);
	
	%% Create Hotmail e-mail button
	pbtMailG=pbtMailIIT;
	pbtMailG.HRef='mailto:marcelo.j.p.alcocer@gmail.com';
	pbtMailG.String=sprintf('<html><a href="%s">marcelo.j.p.alcocer@gmail.com</a></html>',pbtMailG.HRef);
	pbtMailG.Position=[pbtMailG.Position(1)+0.202,pbtMailG.Position(2),0.255,pbtMailG.Position(4)];
	handles_about.pbtMailG=trplGUI_figGUI_CreateJButton(pbtMailG);
	
	%% Create footer
	handles_about.axFooter=axes('Parent',handles_about.figGUIAbout,'Units','normalized','OuterPosition',[0,0,footer_width,footer_height],'Position',[0,0,footer_width,footer_height],'Color',color_uibg,'XColor',color_uibg,'YColor',color_uibg,'XTick',[],'YTick',[]);
	handles_about.atxtboxFooter=annotation(handles_about.figGUIAbout,'textbox',[0,0,footer_width,footer_height],'HorizontalAlignment','center','VerticalAlignment','bottom','EdgeColor','none','FontSize',8);
	atxtboxFooter=get(handles_about.atxtboxFooter);
	atxtboxFooter.String={sprintf('{\\fontsize{7}\\color[rgb]{%f,%f,%f}Copyright  2011 M.J.P. Alcocer: GNU General Public License',color_uibg*0.6);
		'trplGUI makes use of findjobj, Copyright  2010 Yair M. Altman [altmany(at)gmail.com]}'};
	set(handles_about.atxtboxFooter,'String',atxtboxFooter.String);
	
	%% Create homepage button
	pbtHomepage.Parent=handles_about.figGUIAbout;
	pbtHomepage.HRef='http://code.google.com/p/ma-code/';
	pbtHomepage.String=sprintf('<html><a href="%s">Homepage</a></html>',pbtHomepage.HRef);
	pbtHomepage.BackgroundColor=color_uibg;
	pbtHomepage.Units='Normalized';
	pbtHomepage.Position=[0.1750,0.1636,0.095,0.0664];
	handles_about.pbtHomepage=trplGUI_figGUI_CreateJButton(pbtHomepage);
	
	%% Create bug report button
	pbtBugs=pbtHomepage;
	pbtBugs.HRef='http://code.google.com/p/ma-code/issues/list';
	pbtBugs.String=sprintf('<html><a href="%s">Bug Reporting</a></html>',pbtBugs.HRef);
	pbtBugs.Position=[pbtBugs.Position(1)+0.27,pbtBugs.Position(2),0.13,pbtBugs.Position(4)];
	handles_about.pbtBugs=trplGUI_figGUI_CreateJButton(pbtBugs);
	
	%% Create licensing button
	pbtLicence=pbtHomepage;
	pbtLicence.HRef='COPYING.txt';
	pbtLicence.String=sprintf('<html><a href="%s">Licensing</a></html>',pbtLicence.HRef);
	pbtLicence.Position=[pbtLicence.Position(1)+0.57,pbtLicence.Position(2),0.10,pbtLicence.Position(4)];
	handles_about.pbtLicence=trplGUI_figGUI_CreateJButton(pbtLicence);
	
	
end

function trplGUI_menExpProf_Callback(hObject, eventdata, handles, prof_type, exp_type, flag_norm)

	%% Get appdata
	meas=getappdata(gcbf,'meas');
	sels=getappdata(gcbf,'sels');
	
	%% Determine profile type
	switch(lower(prof_type))
		case 'spectrum'
			
			%% Get spectrum
			prof=(trplGetSpec(meas.data,meas.t,sels.ts))';			
			data_indep=meas.wl;
			
			header_indep=['Wavelength [',meas.stat.Scaling.ScalingXUnit,']'];
			str_indep='wl';
			str_prof='spec';
			
		case 'kinetic'
			
			%% Get Kinetic
			prof=trplGetKin(meas.data,meas.wl,sels.wls);
			data_indep=meas.t;
			
			header_indep=['Time [',meas.stat.Scaling.ScalingYUnit,']'];
			str_indep='t';
			str_prof='kin';
			
		otherwise
			throw(MException('trplGUI:ProfExp:ProfType','Profile type not recognised. Expected ''spectrum'' or ''kinetic''. Received ''%s''\n',prof_type));
	end
	
	%% Normalise
	if(flag_norm)
		prof=prof./(repmat(max(abs(prof),[],1),size(prof,1),1));
	end
	

	%% Determine export type
	switch(lower(exp_type))
		case 'ascii'
			
			%% File selection
			exts={'*.txt','Text file (*.txt)';'*.*','All files (*.*)'};
			[file_path,file_name]=fileparts(meas.path);
			[file_name,file_path]=uiputfile(exts,'Export profile',[file_path,filesep,file_name,'_',str_prof,'.txt']);

			%% Return on bad file name or cancel
			if((~ischar(file_name)) || (~ischar(file_path)))
				return
			end

			%% Open file
			fid=fopen([file_path,file_name],'wt');																			% N.b.: wt avoid newline OS issues
			if(fid<0)
				throw(MException('trplGUI:ProfExp:Open','Error opening file %s for writing\n',[file_path,file_name]));
			end
			
			%% Write to file
			fprintf(fid,'%s\t%s\n',header_indep,'PL Intensity [-]');
			fprintf(fid,'%f\t%f\n',[data_indep';prof']);
			
			%% Close file
			fclose(fid);
			
		case 'copy'
			
			%% Copy to clipboard
			clipboard('copy',[sprintf('%s\t%s\n',header_indep,'PL Intensity [-]'),sprintf('%f\t%f\n',[data_indep';prof'])]);
			
		case 'workspace'
			
			%% Generate default variable name
			[file_name,file_name]=fileparts(meas.path);
			var=genvarname([file_name,'_',str_prof]);												% N.b. export2wsdlg handles naming conflicts
			
			%% Open export to workspace dialog
			h_ex2wsdlg=export2wsdlg({'Variable Name:'},{var},{struct(str_indep,data_indep,'pl',prof)},sprintf('Export %s to workspace',lower(prof_type)));
			
			%% Minor cosmetic surgery
			h_pbut_OK=findobj(get(h_ex2wsdlg,'Children'),'flat','String','OK');
			set(h_pbut_OK,'FontWeight','normal');
			uicontrol(h_pbut_OK);
			h_pbut_etxt=findobj(get(h_ex2wsdlg,'Children'),'flat','Style','edit');
			set(findjobj(h_pbut_etxt),'FocusGainedCallback',@trplGUI_etxt_FocusGainedCallback);
			
			%% Depreciated - now use export2wsdlg
% 			var=genvarname(str_prof,evalin('base','who'));																	
% 			assignin('base',var,struct(str_indep,data_indep,'pl',prof));
			
		otherwise
			throw(MException('trplGUI:ProfExp:ExpType','Export type not recognised. Expected ''ascii'', ''copy'' or ''workspace''. Received ''%s''\n',exp_type));
	end
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_menExpData_Callback(hObject, eventdata, handles, exp_type, flag_norm)

	%% Get appdata
	meas=getappdata(gcbf,'meas');

	%% Normalise
	if(flag_norm)
		meas.data=meas.data./max(max(meas.data));
	end

	%% Determine export type
	switch(lower(exp_type))
		case 'ascii'
			
			%% File selection
			exts={'*.txt','Text file (*.txt)';'*.*','All files (*.*)'};
			[file_path,file_name]=fileparts(meas.path);
			[file_name,file_path]=uiputfile(exts,'Export data',[file_path,filesep,file_name,'.txt']);

			%% Return on bad file name or cancel
			if((~ischar(file_name)) || (~ischar(file_path)))
				return
			end

			%% Open file
			fid=fopen([file_path,file_name],'wt');
			if(fid<0)
				throw(MException('trplGUI:MainExp:Open','Error opening file %s for writing\n',[file_path,file_name]));
			end

			%% Write header line
			fprintf(fid,'%s\t',[meas.stat.Scaling.ScalingYUnit,'|',meas.stat.Scaling.ScalingXUnit]);

			%% Close file
			fclose(fid);

			%% Determine newline
			if(ispc())
				newline='pc';
			else
				newline='unix';
			end

			%% Write data
			dlmwrite([file_path,file_name],meas.wl','-append','delimiter','\t','newline',newline);
			dlmwrite([file_path,file_name],[meas.t,meas.data],'-append','delimiter','\t','newline',newline);
			
		case 'copy'
			
			%% Copy to clipboard
			str_format=[repmat('%f\t',1,size(meas.data,2)-1),'%f\n'];
			clipboard('copy',[sprintf(['%s\t',str_format],[meas.stat.Scaling.ScalingYUnit,'|',meas.stat.Scaling.ScalingXUnit],meas.wl),sprintf(['%f\t',str_format],[meas.t,meas.data]')]);
			
		case 'workspace'
			
			%% Generate default variable name
			[file_name,file_name]=fileparts(meas.path);
			var=genvarname(file_name);												% N.b. export2wsdlg handles naming conflicts
			
			%% Open export to workspace dialog
			h_ex2wsdlg=export2wsdlg({'Variable Name:'},{var},{struct('data',meas.data,'wl',meas.wl,'t',meas.t)},'Export data to workspace');
			
			%% Minor cosmetic surgery
			h_pbut_OK=findobj(get(h_ex2wsdlg,'Children'),'flat','String','OK');
			set(h_pbut_OK,'FontWeight','normal');
			uicontrol(h_pbut_OK);
			h_pbut_etxt=findobj(get(h_ex2wsdlg,'Children'),'flat','Style','edit');
			set(findjobj(h_pbut_etxt),'FocusGainedCallback',@trplGUI_etxt_FocusGainedCallback);

			%% Depreciated - now use export2wsdlg
% 			var=genvarname('meas',evalin('base','who'));
% 			evalin('base',[var,'.data=',mat2str(meas.data),';']);
% 			evalin('base',[var,'.wl=',mat2str(meas.wl),';']);
% 			evalin('base',[var,'.t=',mat2str(meas.t),';']);
			
		otherwise
			throw(MException('trplGUI:DataExp:ExpType','Export type not recognised. Expected ''ascii'', ''copy'' or ''workspace''. Received ''%s''\n',exp_type));
	end
	
	%% Update handles
	guidata(hObject,handles);

end



%%-- trplGUI_cmen* --%%

function trplGUI_cmenProf_Callback(hObject, eventdata, handles)

	%% Enable context menu items if selection exists
	if(~isempty(getappdata(gcbf,'sels')))
		
		set(get(hObject,'Children'),'Enable','On');
		
		%% Disable export to workspace if binary distribution
		if(isdeployed())
			set(findobj(get(hObject,'Children'),'flat','-regexp','Tag','cmenProf(Spec|Kin)ExpWS(N)?'),'Enable','Off');
		end
		
	else
		set(get(hObject,'Children'),'Enable','Off');
	end
	
	%% Enable log scaling
	set([handles.cmenLogKin,handles.cmenLogSpec],'Enable','On');
	
	%% Update handles
	guidata(hObject,handles);
	
end

function trplGUI_cmenData_Callback(hObject, eventdata, handles)

	%% Enable context menu items if measurement exists
	if(isappdata(gcbf,'meas'))
		
		set(get(hObject,'Children'),'Enable','On');
		
		%% Disable export to workspace if binary distribution
		if(isdeployed())
			set([handles.cmenDataExpWS,handles.cmenDataExpWSN],'Enable','Off');
		end
		
	else
		set(get(hObject,'Children'),'Enable','Off');
	end
	
	%% Update handles
	guidata(hObject,handles);

end

function trplGUI_cmenCB_Callback(hObject, eventdata, handles)

	%% Enable set full range if measurement exists
	if(isappdata(gcbf,'meas'))
		set(handles.cmenCBFull,'Enable','On');
	else
		set(handles.cmenCBFull,'Enable','Off');
	end
	
	%% Update handles
	guidata(hObject,handles);

end



%%-- trplGUI_tool* --%%

function trplGUI_toolZoomIn_ClickedCallback(hObject, eventdata, handles)

	%% Default zoom in action
	putdowntext('zoomin',gcbo);

end

function trplGUI_toolZoomOut_ClickedCallback(hObject, eventdata, handles)

	%% Default zoom out action
	putdowntext('zoomout',gcbo);

end

function trplGUI_toolPan_ClickedCallback(hObject, eventdata, handles)

	%% Default pan action
	putdowntext('pan',gcbo);

end

function trplGUI_toolDataCursor_ClickedCallback(hObject, eventdata, handles)

	%% Default data cursor action
	putdowntext('datatip',gcbo);

end



%%-- trplGUICorrT0* --%%

function trplGUICorrT0_etxtInt_Callback(hObject, eventdata, handles)
		
	%% Determine edit box type from tag
	ind_int=regexp(get(hObject,'Tag'),'(?<=etxtInt)\d*','match');
	if(isempty(ind_int))
		throw(MException('trplGUI:etxtInt:BadTag','Tag value was not recognised. Valid tags: etxtInt\d*. Received: %s',get(hObject,'Tag')));
	end
	ind_int=uint8(sscanf(ind_int{1},'%d'));								% 8 bit unsigned should be enough

	%% Get new value
	value=str2double(get(hObject,'String'));
	
	%% Get corrections appdata
	corrs=getappdata(handles.figGUI,'corrs');

	%% Bad value - reset to previous value
	if(~isfinite(value))
		
		%% Reset to previous value
		format='%4.3f';													% Add to options
		set(hObject,'String',num2str(corrs.t0.pts(ind_int).t,format));
		
	%% Good value - modify selection
	else
		
		%% Move interval t0 point and line
		corrs.t0.pts(ind_int).t=value;
		set(corrs.t0.lines(ind_int),'YData',[value,value]);
		
		%% Move average t0 line
		avg=mean([corrs.t0.pts.t]);
		set(corrs.t0.avg,'t1',avg,'t2',avg);
		
	end

end

function trplGUICorrT0_etxtAvg_Callback(hObject, eventdata, handles)

	%% Get new value
	value=str2double(get(hObject,'String'));

	%% Get corrections appdata
	corrs=getappdata(handles.figGUI,'corrs');
	
	%% Bad value - reset to previous value
	if(~isfinite(value))
		
		%% Reset to previous value
		format='%4.3f';													% Add to options
		set(hObject,'String',num2str(corrs.t0.avg.t1,format));
		
	%% Good value - modify selection
	else
		set(corrs.t0.avg,'t1',value,'t2',value);
	end

end

function trplGUICorrT0_pbutOk_Callback(hObject, eventdata, handles)

	%% Get corrections and measurement appdata
	meas=getappdata(handles.figGUI,'meas');
	corrs=getappdata(handles.figGUI,'corrs');
	
	%% Translate temporal axis
	meas.t=meas.t-corrs.t0.avg.t1;
	meas.stat.trplGUI.CorrT0=sprintf('%4.3f',corrs.t0.avg.t1);
	setappdata(handles.figGUI,'meas',meas);
	
	%% Translate selection data
	if(~isempty(corrs.t0.sel))
		corrs.t0.sel=corrs.t0.sel-[0,corrs.t0.avg.t1;0,corrs.t0.avg.t1];
		setappdata(handles.figGUI,'corrs',corrs);
	end
	
	%% Set undo/redo
	udrd.Name='t0 Correction';
	udrd.InverseFunction=@trplGUI_menCorrT0_UdRd;
	udrd.InverseVarargin={guidata(handles.figGUI),-corrs.t0.avg.t1};
	udrd.Function=@trplGUI_menCorrT0_UdRd;
	udrd.Varargin={guidata(handles.figGUI),corrs.t0.avg.t1};
	uiundo(handles.figGUI,'function',udrd);
	
	%% Smudge
	trplGUI_figGUI_Dirty(handles.figGUI,true);
	
	%% Replot data
	trplGUI_figGUI_Replot(handles.figGUI);
	
	%% Close figCorT0
	close(handles.figCorrT0);
	
end

function trplGUICorrT0_Close(hObject, eventdata, handles)

	%% Delete figCorT0
	delete(handles.figCorrT0);
	
	%% Get corrections and selections appdata
	corrs=getappdata(handles.figGUI,'corrs');
	
	%% Delete plot objects
	trplGUI_figGUI_Replot(handles.figGUI);
	
	%% Reset selections
	if(~isempty(corrs.t0.sel))
		sels=trplGUIClasses.Sel(corrs.t0.sel(1,:),corrs.t0.sel(2,:),handles.figGUI);
		setappdata(handles.figGUI,'sels',sels);
	end
	
	%% Reset mouse down function
	set(handles.figGUI,'WindowButtonDownFcn',{@trplGUI_figGUI_WindowButtonDownSel_Callback,guidata(handles.figGUI)});
	
	%% Enable trplGUI controls which can change data
	set([handles.menFileOpen,handles.toolOpen],'Enable','on');
	
	%% Remove figCorrT0 from handles
	handles=rmfield(handles,'figCorrT0');
	guidata(handles.figGUI,handles);
	
	%% Remove t0 corrections appdata
	corrs.t0=[];
	setappdata(handles.figGUI,'corrs',corrs);

end

function trplGUICorrT0_KeyPress_Callback(hObject, eventdata, handles)

	%% Handle key presses
	switch(eventdata.Key)
		case 'escape'
			close(handles.figCorrT0);
% 		case 'enter'
% 			trplGUICorrT0_pbutOk_Callback(hObject,[],guidata(handles.figGUI));
	end

end



%%-- trplGUICorrCurv* --%%

function trplGUICorrCurv_etxtInt_Callback(hObject, eventdata, handles)
		
	%% Determine edit box type from tag
	ind_int=regexp(get(hObject,'Tag'),'(?<=etxtInt)\d*','match');
	if(isempty(ind_int))
		throw(MException('trplGUI:etxtInt:BadTag','Tag value was not recognised. Valid tags: etxtInt\d*. Received: %s',get(hObject,'Tag')));
	end
	ind_int=uint8(sscanf(ind_int{1},'%d'));								% 8 bit unsigned should be enough

	%% Get new value
	value=str2double(get(hObject,'String'));
	
	%% Get corrections appdata
	corrs=getappdata(handles.figGUI,'corrs');

	%% Bad value - reset to previous value
	if(~isfinite(value))
		
		%% Reset to previous value
		format='%4.3f';													% Add to options
		set(hObject,'String',num2str(corrs.curv.t0.pts(ind_int).t,format));
		
	%% Good value - modify selection
	else
		
		%% Move interval t0 point and line
		corrs.curv.t0.pts(ind_int).t=value;
		set(corrs.curv.t0.lines(ind_int),'YData',[value,value]);
		
		%% Refit parabola
		final.coeffs=[ones(length(corrs.curv.t0.pts),1),[corrs.curv.t0.pts.wl]',([corrs.curv.t0.pts.wl]').^2]\([corrs.curv.t0.pts.t]');		% mldivide lsq solution
		final.t=[ones(3,1),corrs.curv.parab.wls',(corrs.curv.parab.wls').^2]*final.coeffs;
		set(corrs.curv.parab,'t1',final.t(1),'t2',final.t(2),'t3',final.t(3));
		
	end

end

function trplGUICorrCurv_etxtParab_Callback(hObject, eventdata, handles)

	%% Determine edit box type from tag
	prop=regexp(get(hObject,'Tag'),'(?<=etxtParab)(Wl|T)(1|2|3)','match');
	if(isempty(prop))
		throw(MException('trplGUI:etxtParab:BadTag','Tag value was not recognised. Valid tags: etxtParab(Wl|T)(1|2|3). Received: %s',get(hObject,'Tag')));
	end
	prop=lower(prop{1});

	%% Get new value
	value=str2double(get(hObject,'String'));

	%% Get corrections appdata
	corrs=getappdata(handles.figGUI,'corrs');
	
	%% Bad value - reset to previous value
	if(~isfinite(value))
		
		%% Reset to previous value
		format='%4.3f';													% Add to options
		set(hObject,'String',num2str(corrs.curv.parab.(prop),format));
		
	%% Good value - modify selection
	else
		set(corrs.curv.parab,prop,value);
	end

end

function trplGUICorrCurv_pbutOk_Callback(hObject, eventdata, handles)

	%% Get corrections and measurement appdata
	meas=getappdata(handles.figGUI,'meas');
	corrs=getappdata(handles.figGUI,'corrs');
	
	%% Perform curvature correction
	old_data=meas.data;
	meas.data=trplCurCor(meas.data,meas.wl,meas.t,[corrs.curv.parab.wls;corrs.curv.parab.ts]');
	pt_str=sprintf('%f,%f;',[corrs.curv.parab.wls;corrs.curv.parab.ts]);
	meas.stat.trplGUI.CorrCurv=pt_str;
	if(meas.ITEX)
		meas.stat.Acquisition.CurveCorr='-1';
	end
	setappdata(handles.figGUI,'meas',meas);

	%% Set undo/redo
	udrd.Name='Curvature Correction';
	udrd.InverseFunction=@trplGUI_menCorrCurv_UdRd;
	udrd.InverseVarargin={guidata(handles.figGUI),old_data};
	udrd.Function=@trplGUI_menCorrCurv_UdRd;
	udrd.Varargin={guidata(handles.figGUI),meas.data,pt_str};
	uiundo(handles.figGUI,'function',udrd);
	
	%% Smudge
	trplGUI_figGUI_Dirty(handles.figGUI,true);
	
	%% Replot data
	trplGUI_figGUI_Replot(handles.figGUI);
	
	%% Close figCorCurv
	close(handles.figCorrCurv);
	
end

function trplGUICorrCurv_Close(hObject, eventdata, handles)

	%% Delete figCorrCurv
	delete(handles.figCorrCurv);
	
	%% Get corrections and selections appdata
	corrs=getappdata(handles.figGUI,'corrs');
	
	%% Delete plot objects
	trplGUI_figGUI_Replot(handles.figGUI);
	
	%% Reset selections
	if(~isempty(corrs.curv.sel))
		sels=trplGUIClasses.Sel(corrs.curv.sel(1,:),corrs.curv.sel(2,:),handles.figGUI);
		setappdata(handles.figGUI,'sels',sels);
	end
	
	%% Reset mouse down function
	set(handles.figGUI,'WindowButtonDownFcn',{@trplGUI_figGUI_WindowButtonDownSel_Callback,guidata(handles.figGUI)});
	
	%% Enable trplGUI controls which can change data
	set([handles.menFileOpen,handles.toolOpen],'Enable','on');
	
	%% Remove figCorrCurv from handles
	handles=rmfield(handles,'figCorrCurv');
	guidata(handles.figGUI,handles);
	
	%% Remove curvature correction appdata
	corrs.curv=[];
	setappdata(handles.figGUI,'corrs',corrs);

end

function trplGUICorrCurv_KeyPress_Callback(hObject, eventdata, handles)

	%% Handle key presses
	switch(eventdata.Key)
		case 'escape'
			close(handles.figCorrCurv);
% 		case 'enter'
% 			trplGUICorrT0_pbutOk_Callback(hObject,[],guidata(handles.figGUI));
	end

end


%%-- trplGUICorSubRef* --%%

function trplGUICorrSubRef_pbutFilesel_Callback(hObject, eventdata, handles)

	%% Get measurement appdata
	meas=getappdata(handles.figGUI,'meas');

	%% File selection
	exts={'*.img','TRPL - ITEX data (*.img)';'*.dac','TRPL - Calibrated ASCII data (*.dac)';'*.*','All Files (*.*)'};
	[file_name,file_dir]=uigetfile(exts,'Open TRPL data',meas.path);
	if(file_name==0)
		return
	end
	
	%% Populate etxt
	set(findobj(get(handles.figCorrSubRef,'Children'),'Tag','etxtFilesel'),'String',[file_dir,file_name]);
	
	%% Enable pbutOk
	set(findobj(get(handles.figCorrSubRef,'Children'),'Tag','pbutOk'),'Enable','on');

end

function trplGUICorrSubRef_etxtFilesel_Callback(hObject, eventdata, handles)

	%% Toggle pbutOk enable on populated
	if(isempty(get(hObject,'String')))
		set(findobj(get(handles.figCorrSubRef,'Children'),'Tag','pbutOk'),'Enable','off');
	else
		set(findobj(get(handles.figCorrSubRef,'Children'),'Tag','pbutOk'),'Enable','on');
	end

end

function trplGUICorrSubRef_pbutOk_Callback(hObject, eventdata, handles)

	%% Get path
	h_etxt=findobj(get(handles.figCorrSubRef,'Children'),'Tag','etxtFilesel');
	path=get(h_etxt,'String');
	
	%% Try loading reference file
	try
		[ref.data,ref.wl,ref.t,ref.stat]=trplLoad(path);
	catch ME
		errordlg(sprintf('Error opening reference image file (%s).\n%s.',path,ME.message),'trplGUI - Error opening reference image file','modal');
		return
	end
		
	%% Get measurement appdata
	meas=getappdata(handles.figGUI,'meas');
	
	%% Assert reference image properties
	if(~all(size(ref.data)==size(meas.data)))
		errordlg(sprintf('Reference image dimensions do not match currently loaded data. Loaded dimensions: (%d,%d). Reference dimensions: (%d,%d)',size(meas.data,1),size(meas.data,2),size(ref.data,1),size(ref.data,2)),'trplGUI - Incompatible reference image dimensions','modal');
		return
	end
	
	%% Assert measurement parameters
	if(meas.ITEX && isfield(ref.stat,'Application'))
		meas.stat.Scaling=rmfield(meas.stat.Scaling,{'ScalingXScalingFile','ScalingYScalingFile'});
		meas.stat=rmfield(meas.stat,{'Application','DisplayLUT','Comment'});
		ref.stat.Scaling=rmfield(ref.stat.Scaling,{'ScalingXScalingFile','ScalingYScalingFile'});
		ref.stat=rmfield(ref.stat,{'Application','DisplayLUT','Comment'});
	end
	[neq.section.names,neq.section.meas,neq.section.ref]=fldcmp(meas.stat,ref.stat,'~=');
	
	%% Warn on inconsistent measurement parameters
	if(~isempty(neq.section.names))
		
		%% Determine inconsistencies
		try
			[neq.property.names,neq.property.meas,neq.property.ref]=cellfun(@(x,y) fldcmp(x,y,'~='),neq.section.meas,neq.section.ref,'UniformOutput',false);
		catch ME
			errordlg(sprintf('Number of measurement parameters for the current and reference images appear to be different. This is indicative of different acquisition modes and therefore incompatible data.\n\nError in fldcmp.m:\n\n%s',ME.message),'trplGUI - Reference Image Parameter Number','modal');
			return
		end
		
		%% Define figure layout
		fig.margins=struct('top',7.5,'bottom',0,'left',10,'right',10);
		icon=struct('margins',struct('top',0,'bottom',0,'left',0,'right',20),'dims',[50,50]);
		stxt=struct('margins',struct('top',10,'bottom',10,'left',0,'right',0),'dims',[400,40]);
		table.margins=struct('top',0,'bottom',0,'left',0,'right',10);
		pbuts=struct('margins',struct('horizontal',0,'vertical',7.5),'dims',[69,22.75],'spacing',10);

		%% Create figure
		fig.handle=figure('Color',get(handles.figGUI,'Color'),'DockControls','off','HandleVisibility','callback','IntegerHandle','off','NumberTitle','off','Name','trplGUI - Reference Image Parameter Inconsistencies','Units','pixels','Menubar','none','Toolbar','none','WindowStyle','modal','Visible','on');

		%% Create table
		table.handle=uitable('Parent',fig.handle,'Tag','tabNEq','Units','pixels','RowName',vertcat(neq.property.names{:}),'ColumnName',{'Current','Reference'},'Data',[vertcat(neq.property.meas{:}),vertcat(neq.property.ref{:})]);
		table.extent=get(table.handle,'extent');
		table.position=get(table.handle,'position');
		table.position=[fig.margins.left+icon.dims(1)+icon.margins.right,fig.margins.bottom+(2*pbuts.margins.vertical)+pbuts.dims(2),table.extent(3:4)];
		set(table.handle,'position',table.position);

		%% Resize figure
		fig.position=get(fig.handle,'Position');
		fig.position(3)=fig.margins.left+fig.margins.right+icon.dims(1)+icon.margins.right+max(stxt.dims(1),table.position(3));
		fig.position(4)=fig.margins.top+fig.margins.bottom+stxt.dims(2)+stxt.margins.bottom+stxt.margins.top+table.position(4)+(2*pbuts.margins.vertical)+pbuts.dims(2);
		set(fig.handle,'Position',fig.position);

		%% Create push-buttons
		pbutCancel.position=[fig.position(3)-fig.margins.right-pbuts.margins.horizontal-pbuts.dims(1),fig.margins.bottom+pbuts.margins.vertical,pbuts.dims];
		pbutCancel.handle=uicontrol('Style','pushbutton','Parent',fig.handle,'Tag','pbutCancel','String','Cancel','Units','pixels','Position',pbutCancel.position,'Callback',{@(x,y) uiresume(gcbf)});
		pbutOk.position=[pbutCancel.position(1)-pbuts.spacing-pbuts.dims(1),pbutCancel.position(2),pbuts.dims];
		pbutOk.handle=uicontrol('Style','pushbutton','Parent',fig.handle,'Tag','pbutOk','String','OK','Units','pixels','Position',pbutOk.position,'Callback',{@(x,y) uiresume(gcbf)});

		%% Create static text
		stxt.position=[table.position(1),table.position(2)+table.position(4)+stxt.margins.bottom,stxt.dims];
		stxt.handle=uicontrol('Style','text','Parent',fig.handle,'Tag','stxtQuest','Position',stxt.position,'HorizontalAlignment','left','String','The measurement parameters for the current and reference images appear to be different. Do you wish to continue with reference image subtraction?');

		%% Create icon
		icon.position=[fig.margins.left,fig.position(4)-fig.margins.top-icon.dims(2),icon.dims];
		icon.handle=axes('Parent',fig.handle,'Tag','axIcon','Units','pixels','Position',icon.position,'Visible','off');
		dlgicons=load('dialogicons.mat');
		icon.data=dlgicons.warnIconData;
		icon.cmap=dlgicons.warnIconMap;
		icon.cmap(256,:)=get(fig.handle,'Color');					% Set background
		icon.img=image('Parent',icon.handle,'CData',icon.data);
		set(fig.handle,'Colormap',icon.cmap);
		set(icon.handle,'XLim',get(icon.img,'XData')+[-0.5,0.5],'YLim',get(icon.img,'YData')+[-0.5,0.5],'YDir','reverse');

		%% Show figure
% 		set(fig.handle,'Visible','on');												% Hiding figure causes display problems with uitable
		
		%% Wait for user choice
		uicontrol(pbutCancel.handle);
		uiwait(fig.handle);
		choice=get(gco,'Tag');
		close(fig.handle);
		
		%% Return on cancel
		if(strcmp(choice,'pbutCancel'))
			return
		end
		
	end
	
	%% Reload measurement appdata
	meas=getappdata(handles.figGUI,'meas');											% Reloads removed stat fields
	
	%% Perform subtraction
	meas.data=meas.data-ref.data;
	h_chkbox=findobj(get(handles.figCorrSubRef,'Children'),'Tag','chkboxClip');
	clip=struct('flag',false,'ind',[],'val',[]);
	if(get(h_chkbox,'Value')==get(h_chkbox,'Max'))
		clip.flag=true;
		clip.ind=find(meas.data<0);
		clip.val=meas.data(clip.ind);
		meas.data(clip.ind)=0;
	end
	meas.stat.trplGUI.CorrSubRef=[path,'|',sprintf('%d',clip.flag)];
	if(meas.ITEX)
		meas.stat.Acquisition.BacksubCorr='-1';
	end
	setappdata(handles.figGUI,'meas',meas);
	
	%% Set undo/redo
	udrd.Name='Reference Image Subtraction';
	udrd.InverseFunction=@trplGUI_menCorrSubRef_UdRd;
	udrd.InverseVarargin={guidata(handles.figGUI),path,clip};
	udrd.Function=@trplGUI_menCorrSubRef_UdRd;
	udrd.Varargin={guidata(handles.figGUI),path,clip};
	uiundo(handles.figGUI,'function',udrd);
	
	%% Smudge
	trplGUI_figGUI_Dirty(handles.figGUI,true);
	
	%% Replot data
	trplGUI_figGUI_Replot(handles.figGUI);
	
	%% Close figCorrSubRef
	close(handles.figCorrSubRef);
	
end

function trplGUICorrSubRef_Close(hObject, eventdata, handles)

	%% Delete figCorrSubRef
	delete(handles.figCorrSubRef);
	
	%% Enable trplGUI controls which can change data
	set([handles.menFileOpen,handles.toolOpen],'Enable','on');
	
	%% Remove figCorrSubRef from handles
	handles=rmfield(handles,'figCorrSubRef');
	guidata(handles.figGUI,handles);

end

function trplGUICorrSubRef_KeyPress_Callback(hObject, eventdata, handles)

	%% Handle key presses
	switch(eventdata.Key)
		case 'escape'
			close(handles.figCorrSubRef);
		case 'enter'
			trplGUICorrT0_pbutOk_Callback(hObject,[],guidata(handles.figGUI));
	end

end
